# (c) 2005 Ben Bangert
# This module is part of the Python Paste Project and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
from paste.fixture import TestApp
from paste.request import get_cookie_dict
from paste.wsgiwrappers import WSGIRequest

def simpleapp(environ, start_response):
    status = '200 OK'
    response_headers = [('Content-type','text/plain')]
    start_response(status, response_headers)
    request = WSGIRequest(environ)
    body = [
        'Hello world!\n', 'The get is %s' % str(request.GET),
        ' and Val is %s\n' % request.GET.get('name'),
        'The languages are: %s\n' % request.languages,
        'The accepttypes is: %s\n' % request.match_accept(['text/html', 'application/xml'])]
    body = [line.encode('utf8')  for line in body]
    return body

def test_gets():
    app = TestApp(simpleapp)
    res = app.get('/')
    assert 'Hello' in res
    assert "get is MultiDict([])" in res

    res = app.get('/?name=george')
    res.mustcontain("get is MultiDict([('name', 'george')])")
    res.mustcontain("Val is george")

def test_language_parsing():
    app = TestApp(simpleapp)
    res = app.get('/')
    assert "The languages are: ['en-us']" in res

    res = app.get('/', headers={'Accept-Language':'da, en-gb;q=0.8, en;q=0.7'})
    assert "languages are: ['da', 'en-gb', 'en', 'en-us']" in res

    res = app.get('/', headers={'Accept-Language':'en-gb;q=0.8, da, en;q=0.7'})
    assert "languages are: ['da', 'en-gb', 'en', 'en-us']" in res

def test_mime_parsing():
    app = TestApp(simpleapp)
    res = app.get('/', headers={'Accept':'text/html'})
    assert "accepttypes is: ['text/html']" in res

    res = app.get('/', headers={'Accept':'application/xml'})
    assert "accepttypes is: ['application/xml']" in res

    res = app.get('/', headers={'Accept':'application/xml,*/*'})
    assert "accepttypes is: ['text/html', 'application/xml']" in res

def test_bad_cookie():
    env = {}
    env['HTTP_COOKIE'] = '070-it-:><?0'
    assert get_cookie_dict(env) == {}
    env['HTTP_COOKIE'] = 'foo=bar'
    assert get_cookie_dict(env) == {'foo': 'bar'}
    env['HTTP_COOKIE'] = '...'
    assert get_cookie_dict(env) == {}
    env['HTTP_COOKIE'] = '=foo'
    assert get_cookie_dict(env) == {}
    env['HTTP_COOKIE'] = '?='
    assert get_cookie_dict(env) == {}



# (c) 2005 Ben Bangert
# This module is part of the Python Paste Project and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

import pytest

from paste.fixture import TestApp
from paste.registry import (
    Registry,
    RegistryManager,
    StackedObjectProxy,
    restorer,
)
from paste.evalexception.middleware import EvalException

regobj = StackedObjectProxy()
secondobj = StackedObjectProxy(default=dict(hi='people'))

def simpleapp(environ, start_response):
    status = '200 OK'
    response_headers = [('Content-type','text/plain')]
    start_response(status, response_headers)
    return [b'Hello world!\n']

def simpleapp_withregistry(environ, start_response):
    status = '200 OK'
    response_headers = [('Content-type','text/plain')]
    start_response(status, response_headers)
    body = 'Hello world!Value is %s\n' % regobj.keys()
    body = body.encode('utf8')
    return [body]

def simpleapp_withregistry_default(environ, start_response):
    status = '200 OK'
    response_headers = [('Content-type','text/plain')]
    start_response(status, response_headers)
    body = 'Hello world!Value is %s\n' % secondobj
    body = body.encode('utf8')
    return [body]


class RegistryUsingApp:
    def __init__(self, var, value, raise_exc=False):
        self.var = var
        self.value = value
        self.raise_exc = raise_exc

    def __call__(self, environ, start_response):
        if 'paste.registry' in environ:
            environ['paste.registry'].register(self.var, self.value)
        if self.raise_exc:
            raise self.raise_exc
        status = '200 OK'
        response_headers = [('Content-type','text/plain')]
        start_response(status, response_headers)
        body = 'Hello world!\nThe variable is %s' % str(regobj)
        body = body.encode('utf8')
        return [body]

class RegistryUsingIteratorApp:
    def __init__(self, var, value):
        self.var = var
        self.value = value

    def __call__(self, environ, start_response):
        if 'paste.registry' in environ:
            environ['paste.registry'].register(self.var, self.value)
        status = '200 OK'
        response_headers = [('Content-type','text/plain')]
        start_response(status, response_headers)
        body = 'Hello world!\nThe variable is %s' % str(regobj)
        body = body.encode('utf8')
        return iter([body])

class RegistryMiddleMan:
    def __init__(self, app, var, value, depth):
        self.app = app
        self.var = var
        self.value = value
        self.depth = depth

    def __call__(self, environ, start_response):
        if 'paste.registry' in environ:
            environ['paste.registry'].register(self.var, self.value)
        line = ('\nInserted by middleware!\nInsertValue at depth %s is %s'
                % (self.depth, str(regobj)))
        line = line.encode('utf8')
        app_response = [line]
        app_iter = None
        app_iter = self.app(environ, start_response)
        if type(app_iter) in (list, tuple):
            app_response.extend(app_iter)
        else:
            response = []
            for line in app_iter:
                response.append(line)
            if hasattr(app_iter, 'close'):
                app_iter.close()
            app_response.extend(response)
        line = ('\nAppended by middleware!\nAppendValue at \
                depth %s is %s' % (self.depth, str(regobj)))
        line = line.encode('utf8')
        app_response.append(line)
        return app_response


def test_simple():
    app = TestApp(simpleapp)
    response = app.get('/')
    assert 'Hello world' in response

def test_solo_registry():
    obj = {'hi':'people'}
    wsgiapp = RegistryUsingApp(regobj, obj)
    wsgiapp = RegistryManager(wsgiapp)
    app = TestApp(wsgiapp)
    res = app.get('/')
    assert 'Hello world' in res
    assert 'The variable is' in res
    assert "{'hi': 'people'}" in res

def test_registry_no_object_error():
    app = TestApp(simpleapp_withregistry)
    pytest.raises(TypeError, app.get, '/')

def test_with_default_object():
    app = TestApp(simpleapp_withregistry_default)
    res = app.get('/')
    print(res)
    assert 'Hello world' in res
    assert "Value is {'hi': 'people'}" in res

def test_double_registry():
    obj = {'hi':'people'}
    secondobj = {'bye':'friends'}
    wsgiapp = RegistryUsingApp(regobj, obj)
    wsgiapp = RegistryManager(wsgiapp)
    wsgiapp = RegistryMiddleMan(wsgiapp, regobj, secondobj, 0)
    wsgiapp = RegistryManager(wsgiapp)
    app = TestApp(wsgiapp)
    res = app.get('/')
    assert 'Hello world' in res
    assert 'The variable is' in res
    assert "{'hi': 'people'}" in res
    assert "InsertValue at depth 0 is {'bye': 'friends'}" in res
    assert "AppendValue at depth 0 is {'bye': 'friends'}" in res

def test_really_deep_registry():
    keylist = ['fred', 'wilma', 'barney', 'homer', 'marge', 'bart', 'lisa',
        'maggie']
    valuelist = range(0, len(keylist))
    obj = {'hi':'people'}
    wsgiapp = RegistryUsingApp(regobj, obj)
    wsgiapp = RegistryManager(wsgiapp)
    for depth in valuelist:
        newobj = {keylist[depth]: depth}
        wsgiapp = RegistryMiddleMan(wsgiapp, regobj, newobj, depth)
        wsgiapp = RegistryManager(wsgiapp)
    app = TestApp(wsgiapp)
    res = app.get('/')
    assert 'Hello world' in res
    assert 'The variable is' in res
    assert "{'hi': 'people'}" in res
    for depth in valuelist:
        assert "InsertValue at depth %s is {'%s': %s}" % \
            (depth, keylist[depth], depth) in res
    for depth in valuelist:
        assert "AppendValue at depth %s is {'%s': %s}" % \
            (depth, keylist[depth], depth) in res

def test_iterating_response():
    obj = {'hi':'people'}
    secondobj = {'bye':'friends'}
    wsgiapp = RegistryUsingIteratorApp(regobj, obj)
    wsgiapp = RegistryManager(wsgiapp)
    wsgiapp = RegistryMiddleMan(wsgiapp, regobj, secondobj, 0)
    wsgiapp = RegistryManager(wsgiapp)
    app = TestApp(wsgiapp)
    res = app.get('/')
    assert 'Hello world' in res
    assert 'The variable is' in res
    assert "{'hi': 'people'}" in res
    assert "InsertValue at depth 0 is {'bye': 'friends'}" in res
    assert "AppendValue at depth 0 is {'bye': 'friends'}" in res

def _test_restorer(stack, data):
    # We need to test the request's specific Registry. Initialize it here so we
    # can use it later (RegistryManager will re-use one preexisting in the
    # environ)
    registry = Registry()
    extra_environ={'paste.throw_errors': False,
                   'paste.registry': registry}
    request_id = restorer.get_request_id(extra_environ)
    app = TestApp(stack)
    res = app.get('/', extra_environ=extra_environ, expect_errors=True)

    # Ensure all the StackedObjectProxies are empty after the RegistryUsingApp
    # raises an Exception
    for stacked, proxied_obj, test_cleanup in data:
        only_key = list(proxied_obj.keys())[0]
        try:
            assert only_key not in stacked
            assert False
        except TypeError:
            # Definitely empty
            pass

    # Ensure the StackedObjectProxies & Registry 'work' in the simulated
    # EvalException context
    replace = {'replace': 'dict'}
    new = {'new': 'object'}
    restorer.restoration_begin(request_id)
    try:
        for stacked, proxied_obj, test_cleanup in data:
            # Ensure our original data magically re-appears in this context
            only_key, only_val = list(proxied_obj.items())[0]
            assert only_key in stacked and stacked[only_key] == only_val

            # Ensure the Registry still works
            registry.prepare()
            registry.register(stacked, new)
            assert 'new' in stacked and stacked['new'] == 'object'
            registry.cleanup()

            # Back to the original (pre-prepare())
            assert only_key in stacked and stacked[only_key] == only_val

            registry.replace(stacked, replace)
            assert 'replace' in stacked and stacked['replace'] == 'dict'

            if test_cleanup:
                registry.cleanup()
                try:
                    stacked._current_obj()
                    assert False
                except TypeError:
                    # Definitely empty
                    pass
    finally:
        restorer.restoration_end()

def _restorer_data():
    S = StackedObjectProxy
    d = [[S(name='first'), dict(top='of the registry stack'), False],
         [S(name='second'), dict(middle='of the stack'), False],
         [S(name='third'), dict(bottom='of the STACK.'), False]]
    return d

def _set_cleanup_test(data):
    """Instruct _test_restorer to check registry cleanup at this level of the stack
    """
    data[2] = True

def test_restorer_basic():
    data = _restorer_data()[0]
    wsgiapp = RegistryUsingApp(data[0], data[1], raise_exc=Exception())
    wsgiapp = RegistryManager(wsgiapp)
    _set_cleanup_test(data)
    wsgiapp = EvalException(wsgiapp)
    _test_restorer(wsgiapp, [data])

def test_restorer_basic_manager_outside():
    data = _restorer_data()[0]
    wsgiapp = RegistryUsingApp(data[0], data[1], raise_exc=Exception())
    wsgiapp = EvalException(wsgiapp)
    wsgiapp = RegistryManager(wsgiapp)
    _set_cleanup_test(data)
    _test_restorer(wsgiapp, [data])

def test_restorer_middleman_nested_evalexception():
    data = _restorer_data()[:2]
    wsgiapp = RegistryUsingApp(data[0][0], data[0][1], raise_exc=Exception())
    wsgiapp = EvalException(wsgiapp)
    wsgiapp = RegistryMiddleMan(wsgiapp, data[1][0], data[1][1], 0)
    wsgiapp = RegistryManager(wsgiapp)
    _set_cleanup_test(data[1])
    _test_restorer(wsgiapp, data)

def test_restorer_nested_middleman():
    data = _restorer_data()[:2]
    wsgiapp = RegistryUsingApp(data[0][0], data[0][1], raise_exc=Exception())
    wsgiapp = RegistryManager(wsgiapp)
    _set_cleanup_test(data[0])
    wsgiapp = RegistryMiddleMan(wsgiapp, data[1][0], data[1][1], 0)
    wsgiapp = EvalException(wsgiapp)
    wsgiapp = RegistryManager(wsgiapp)
    _set_cleanup_test(data[1])
    _test_restorer(wsgiapp, data)

def test_restorer_middlemen_nested_evalexception():
    data = _restorer_data()
    wsgiapp = RegistryUsingApp(data[0][0], data[0][1], raise_exc=Exception())
    wsgiapp = RegistryManager(wsgiapp)
    _set_cleanup_test(data[0])
    wsgiapp = EvalException(wsgiapp)
    wsgiapp = RegistryMiddleMan(wsgiapp, data[1][0], data[1][1], 0)
    wsgiapp = RegistryManager(wsgiapp)
    _set_cleanup_test(data[1])
    wsgiapp = RegistryMiddleMan(wsgiapp, data[2][0], data[2][1], 1)
    wsgiapp = RegistryManager(wsgiapp)
    _set_cleanup_test(data[2])
    _test_restorer(wsgiapp, data)

def test_restorer_disabled():
    # Ensure restoration_begin/end work safely when there's no Registry
    wsgiapp = TestApp(simpleapp)
    wsgiapp.get('/')
    try:
        restorer.restoration_begin(1)
    finally:
        restorer.restoration_end()
        # A second call should do nothing
        restorer.restoration_end()



# -*- coding: utf-8 -*-
# (c) 2007 Ian Bicking and Philip Jenvey; written for Paste (http://pythonpaste.org)
# Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php
import gc
import io

import pytest

from paste.util.field_storage import FieldStorage
from paste.util.multidict import MultiDict, UnicodeMultiDict

def test_dict():
    d = MultiDict({'a': 1})
    assert d.items() == [('a', 1)]

    d['b'] = 2
    d['c'] = 3
    assert d.items() == [('a', 1), ('b', 2), ('c', 3)]

    d['b'] = 4
    assert d.items() == [('a', 1), ('c', 3), ('b', 4)]

    d.add('b', 5)
    pytest.raises(KeyError, d.getone, "b")
    assert d.getall('b') == [4, 5]
    assert d.items() == [('a', 1), ('c', 3), ('b', 4), ('b', 5)]

    del d['b']
    assert d.items() == [('a', 1), ('c', 3)]
    assert d.pop('xxx', 5) == 5
    assert d.getone('a') == 1
    assert d.popitem() == ('c', 3)
    assert d.items() == [('a', 1)]

    item = []
    assert d.setdefault('z', item) is item
    assert d.items() == [('a', 1), ('z', item)]

    assert d.setdefault('y', 6) == 6

    assert d.mixed() == {'a': 1, 'y': 6, 'z': item}
    assert d.dict_of_lists() == {'a': [1], 'y': [6], 'z': [item]}

    assert 'a' in d
    dcopy = d.copy()
    assert dcopy is not d
    assert dcopy == d
    d['x'] = 'x test'
    assert dcopy != d

    d[(1, None)] = (None, 1)
    assert d.items() == [('a', 1), ('z', []), ('y', 6), ('x', 'x test'),
                         ((1, None), (None, 1))]

def test_unicode_dict():
    _test_unicode_dict()
    _test_unicode_dict(decode_param_names=True)

def _test_unicode_dict(decode_param_names=False):
    d = UnicodeMultiDict(MultiDict({b'a': 'a test'}))
    d.encoding = 'utf-8'
    d.errors = 'ignore'

    if decode_param_names:
        key_str = str
        k = lambda key: key
        d.decode_keys = True
    else:
        key_str = bytes
        k = lambda key: key.encode()

    def assert_unicode(obj):
        assert isinstance(obj, str)

    def assert_key_str(obj):
        assert isinstance(obj, key_str)

    def assert_unicode_item(obj):
        key, value = obj
        assert isinstance(key, key_str)
        assert isinstance(value, str)

    assert d.items() == [(k('a'), u'a test')]
    map(assert_key_str, d.keys())
    map(assert_unicode, d.values())

    d[b'b'] = b'2 test'
    d[b'c'] = b'3 test'
    assert d.items() == [(k('a'), u'a test'), (k('b'), u'2 test'), (k('c'), u'3 test')]
    list(map(assert_unicode_item, d.items()))

    d[k('b')] = b'4 test'
    assert d.items() == [(k('a'), u'a test'), (k('c'), u'3 test'), (k('b'), u'4 test')], d.items()
    list(map(assert_unicode_item, d.items()))

    d.add(k('b'), b'5 test')
    pytest.raises(KeyError, d.getone, k("b"))
    assert d.getall(k('b')) == [u'4 test', u'5 test']
    map(assert_unicode, d.getall('b'))
    assert d.items() == [(k('a'), u'a test'), (k('c'), u'3 test'), (k('b'), u'4 test'),
                         (k('b'), u'5 test')]
    list(map(assert_unicode_item, d.items()))

    del d[k('b')]
    assert d.items() == [(k('a'), u'a test'), (k('c'), u'3 test')]
    list(map(assert_unicode_item, d.items()))
    assert d.pop('xxx', u'5 test') == u'5 test'
    assert isinstance(d.pop('xxx', u'5 test'), str)
    assert d.getone(k('a')) == u'a test'
    assert isinstance(d.getone(k('a')), str)
    assert d.popitem() == (k('c'), u'3 test')
    d[k('c')] = b'3 test'
    assert_unicode_item(d.popitem())
    assert d.items() == [(k('a'), u'a test')]
    list(map(assert_unicode_item, d.items()))

    item = []
    assert d.setdefault(k('z'), item) is item
    items = d.items()
    assert items == [(k('a'), u'a test'), (k('z'), item)]
    assert isinstance(items[1][0], key_str)
    assert isinstance(items[1][1], list)

    assert isinstance(d.setdefault(k('y'), b'y test'), str)
    assert isinstance(d[k('y')], str)

    assert d.mixed() == {k('a'): u'a test', k('y'): u'y test', k('z'): item}
    assert d.dict_of_lists() == {k('a'): [u'a test'], k('y'): [u'y test'],
                                 k('z'): [item]}
    del d[k('z')]
    list(map(assert_unicode_item, d.mixed().items()))
    list(map(assert_unicode_item, [(key, value[0]) for \
                                   key, value in d.dict_of_lists().items()]))

    assert k('a') in d
    dcopy = d.copy()
    assert dcopy is not d
    assert dcopy == d
    d[k('x')] = 'x test'
    assert dcopy != d

    d[(1, None)] = (None, 1)
    assert d.items() == [(k('a'), u'a test'), (k('y'), u'y test'), (k('x'), u'x test'),
                         ((1, None), (None, 1))]
    item = d.items()[-1]
    assert isinstance(item[0], tuple)
    assert isinstance(item[1], tuple)

    fs = FieldStorage()
    fs.name = 'thefile'
    fs.filename = 'hello.txt'
    fs.file = io.BytesIO(b'hello')
    d[k('f')] = fs
    ufs = d[k('f')]
    assert isinstance(ufs, FieldStorage)
    assert ufs.name == fs.name
    assert isinstance(ufs.name, str)
    assert ufs.filename == fs.filename
    assert isinstance(ufs.filename, str)
    assert isinstance(ufs.value, bytes)
    assert ufs.value == b'hello'
    ufs = None
    gc.collect()
    assert not fs.file.closed



from paste.fixture import TestApp
try:
    from paste.debug.profile import ProfileMiddleware, profile_decorator
    disable = False
except ImportError:
    disable = True

if not disable:
    def simple_app(environ, start_response):
        start_response('200 OK', [('content-type', 'text/html')])
        return ['all ok']

    def long_func():
        for i in range(1000):
            pass
        return 'test'

    def test_profile():
        app = TestApp(ProfileMiddleware(simple_app, {}))
        res = app.get('/')
        # The original app:
        res.mustcontain('all ok')
        # The profile information:
        res.mustcontain('<pre')

    def test_decorator():
        value = profile_decorator()(long_func)()
        assert value == 'test'




# Slightly modified tests from the original cgi test module.

# Copyright © 2001-2023 Python Software Foundation; All Rights Reserved.

import sys
import tempfile
from collections import namedtuple
from io import BytesIO

import pytest

from paste.util import field_storage
from paste.util.field_storage import FieldStorage, parse_header


class HackedSysModule:
    # The regression test will have real values in sys.argv, which
    # will completely confuse the test of the field_storage module
    argv = []
    stdin = sys.stdin


field_storage.sys = HackedSysModule()


parse_strict_test_cases = [
    ("", {}),
    ("&", ValueError("bad query field: ''")),
    ("&&", ValueError("bad query field: ''")),
    # Should the next few really be valid?
    ("=", {}),
    ("=&=", {}),
    # This rest seem to make sense
    ("=a", {'': ['a']}),
    ("&=a", ValueError("bad query field: ''")),
    ("=a&", ValueError("bad query field: ''")),
    ("=&a", ValueError("bad query field: 'a'")),
    ("b=a", {'b': ['a']}),
    ("b+=a", {'b ': ['a']}),
    ("a=b=a", {'a': ['b=a']}),
    ("a=+b=a", {'a': [' b=a']}),
    ("&b=a", ValueError("bad query field: ''")),
    ("b&=a", ValueError("bad query field: 'b'")),
    ("a=a+b&b=b+c", {'a': ['a b'], 'b': ['b c']}),
    ("a=a+b&a=b+a", {'a': ['a b', 'b a']}),
    ("x=1&y=2.0&z=2-3.%2b0", {'x': ['1'], 'y': ['2.0'], 'z': ['2-3.+0']}),
    ("Hbc5161168c542333633315dee1182227:key_store_seqid=400006&cuyer=r"
     "&view=bustomer&order_id=0bb2e248638833d48cb7fed300000f1b"
     "&expire=964546263&lobale=en-US&kid=130003.300038&ss=env",
     {'Hbc5161168c542333633315dee1182227:key_store_seqid': ['400006'],
      'cuyer': ['r'],
      'expire': ['964546263'],
      'kid': ['130003.300038'],
      'lobale': ['en-US'],
      'order_id': ['0bb2e248638833d48cb7fed300000f1b'],
      'ss': ['env'],
      'view': ['bustomer'],
      }),

    ("group_id=5470&set=custom&_assigned_to=31392&_status=1"
     "&_category=100&SUBMIT=Browse",
     {'SUBMIT': ['Browse'],
      '_assigned_to': ['31392'],
      '_category': ['100'],
      '_status': ['1'],
      'group_id': ['5470'],
      'set': ['custom'],
      })
    ]


def gen_result(data, environ):
    encoding = 'latin-1'
    fake_stdin = BytesIO(data.encode(encoding))
    fake_stdin.seek(0)
    form = FieldStorage(fp=fake_stdin, environ=environ, encoding=encoding)
    return {k: form.getlist(k) if isinstance(v, list) else v.value
            for k, v in dict(form).items()}


def test_fieldstorage_properties():
    fs = FieldStorage()
    assert not fs
    assert "FieldStorage" in repr(fs)
    assert list(fs) == list(fs.keys())
    fs.list.append(namedtuple('MockFieldStorage', 'name')('fieldvalue'))
    assert fs


def test_fieldstorage_invalid():
    with pytest.raises(TypeError):
        FieldStorage("not-a-file-obj", environ={"REQUEST_METHOD": "PUT"})
    with pytest.raises(TypeError):
        FieldStorage("foo", "bar")
    fs = FieldStorage(headers={'content-type': 'text/plain'})
    with pytest.raises(TypeError):
        bool(fs)


def test_strict():
    for orig, expect in parse_strict_test_cases:
        env = {'QUERY_STRING': orig}
        fs = FieldStorage(environ=env)
        if isinstance(expect, dict):
            # test dict interface
            assert len(expect) == len(fs)
            assert len(expect.keys()) == len(fs.keys())
            assert fs.getvalue("nonexistent field", "default") == "default"
            # test individual fields
            for key in expect.keys():
                expect_val = expect[key]
                assert key in fs
                if len(expect_val) > 1:
                    assert fs.getvalue(key) == expect_val
                else:
                    assert fs.getvalue(key) == expect_val[0]


def test_separator():
    parse_semicolon = [
        ("x=1;y=2.0", {'x': ['1'], 'y': ['2.0']}),
        ("x=1;y=2.0;z=2-3.%2b0", {'x': ['1'], 'y': ['2.0'], 'z': ['2-3.+0']}),
        (";", ValueError("bad query field: ''")),
        (";;", ValueError("bad query field: ''")),
        ("=;a", ValueError("bad query field: 'a'")),
        (";b=a", ValueError("bad query field: ''")),
        ("b;=a", ValueError("bad query field: 'b'")),
        ("a=a+b;b=b+c", {'a': ['a b'], 'b': ['b c']}),
        ("a=a+b;a=b+a", {'a': ['a b', 'b a']}),
    ]
    for orig, expect in parse_semicolon:
        env = {'QUERY_STRING': orig}
        fs = FieldStorage(separator=';', environ=env)
        if isinstance(expect, dict):
            for key in expect.keys():
                expect_val = expect[key]
                assert key in fs
                if len(expect_val) > 1:
                    assert fs.getvalue(key) == expect_val
                else:
                    assert fs.getvalue(key) == expect_val[0]


def test_fieldstorage_readline():
    # FieldStorage uses readline, which has the capacity to read all
    # contents of the input file into memory; we use readline's size argument
    # to prevent that for files that do not contain any newlines in
    # non-GET/HEAD requests
    class TestReadlineFile:
        def __init__(self, file):
            self.file = file
            self.numcalls = 0

        def readline(self, size=None):
            self.numcalls += 1
            if size:
                return self.file.readline(size)
            else:
                return self.file.readline()

        def __getattr__(self, name):
            file = self.__dict__['file']
            a = getattr(file, name)
            if not isinstance(a, int):
                setattr(self, name, a)
            return a

    f = TestReadlineFile(tempfile.TemporaryFile("wb+"))
    try:
        f.write(b'x' * 256 * 1024)
        f.seek(0)
        env = {'REQUEST_METHOD': 'PUT'}
        fs = FieldStorage(fp=f, environ=env)
        try:
            # if we're not chunking properly, readline is only called twice
            # (by read_binary); if we are chunking properly, it will be called
            # 5 times as long as the chunk size is 1 << 16.
            assert f.numcalls >= 2
        finally:
            fs.file.close()
    finally:
        f.close()


def test_fieldstorage_multipart():
    # Test basic FieldStorage multipart parsing
    env = {
        'REQUEST_METHOD': 'POST',
        'CONTENT_TYPE': f'multipart/form-data; boundary={BOUNDARY}',
        'CONTENT_LENGTH': '558'}
    fp = BytesIO(POSTDATA.encode('latin-1'))
    fs = FieldStorage(fp, environ=env, encoding="latin-1")
    assert len(fs.list) == 4
    expect = [{'name': 'id', 'filename': None, 'value': '1234'},
              {'name': 'title', 'filename': None, 'value': ''},
              {'name': 'file', 'filename': 'test.txt',
               'value': b'Testing 123.\n'},
              {'name': 'submit', 'filename': None, 'value': ' Add '}]
    for x in range(len(fs.list)):
        for k, exp in expect[x].items():
            got = getattr(fs.list[x], k)
            assert got == exp


def test_fieldstorage_multipart_leading_whitespace():
    env = {
        'REQUEST_METHOD': 'POST',
        'CONTENT_TYPE': f'multipart/form-data; boundary={BOUNDARY}',
        'CONTENT_LENGTH': '560'}
    # Add some leading whitespace to our post data that will cause the
    # first line to not be the inner boundary.
    fp = BytesIO(b"\r\n" + POSTDATA.encode('latin-1'))
    fs = FieldStorage(fp, environ=env, encoding="latin-1")
    assert len(fs.list) == 4
    expect = [{'name': 'id', 'filename': None, 'value': '1234'},
              {'name': 'title', 'filename': None, 'value': ''},
              {'name': 'file', 'filename': 'test.txt',
               'value': b'Testing 123.\n'},
              {'name': 'submit', 'filename': None, 'value': ' Add '}]
    for x in range(len(fs.list)):
        for k, exp in expect[x].items():
            got = getattr(fs.list[x], k)
            assert got == exp


def test_fieldstorage_multipart_non_ascii():
    # Test basic FieldStorage multipart parsing
    env = {'REQUEST_METHOD': 'POST',
           'CONTENT_TYPE': f'multipart/form-data; boundary={BOUNDARY}',
           'CONTENT_LENGTH': '558'}
    for encoding in ['iso-8859-1', 'utf-8']:
        fp = BytesIO(POSTDATA_NON_ASCII.encode(encoding))
        fs = FieldStorage(fp, environ=env, encoding=encoding)
        assert len(fs.list) == 1
        expect = [{'name': 'id', 'filename': None, 'value': '\xe7\xf1\x80'}]
        for x in range(len(fs.list)):
            for k, exp in expect[x].items():
                got = getattr(fs.list[x], k)
                assert got == exp


def test_fieldstorage_multipart_maxline():
    # Issue #18167
    maxline = 1 << 16 - 1

    def check(content):
        data = """---123
Content-Disposition: form-data; name="upload"; filename="fake.txt"
Content-Type: text/plain

{}
---123--
""".replace('\n', '\r\n').format(content)
        environ = {
            'CONTENT_LENGTH':   str(len(data)),
            'CONTENT_TYPE':     'multipart/form-data; boundary=-123',
            'REQUEST_METHOD':   'POST',
        }

        assert gen_result(data, environ) == {
            'upload': content.encode('latin1')}
    check('x' * maxline)
    check('x' * maxline + '\r')
    check('x' * maxline + '\r' + 'y' * maxline)


def test_fieldstorage_multipart_w3c():
    # Test basic FieldStorage multipart parsing (W3C sample)
    env = {
        'REQUEST_METHOD': 'POST',
        'CONTENT_TYPE': f'multipart/form-data; boundary={BOUNDARY_W3}',
        'CONTENT_LENGTH': str(len(POSTDATA_W3))}
    fp = BytesIO(POSTDATA_W3.encode('latin-1'))
    fs = FieldStorage(fp, environ=env, encoding="latin-1")
    assert len(fs.list) == 2
    assert fs.list[0].name == 'submit-name'
    assert fs.list[0].value == 'Larry'
    assert fs.list[1].name == 'files'
    files = fs.list[1].value
    assert len(files) == 2
    expect = [{'name': None, 'filename': 'file1.txt',
               'value': b'... contents of file1.txt ...'},
              {'name': None, 'filename': 'file2.gif',
               'value': b'...contents of file2.gif...'}]
    for x in range(len(files)):
        for k, exp in expect[x].items():
            got = getattr(files[x], k)
            assert got == exp


def test_fieldstorage_part_content_length():
    boundary = "JfISa01"
    postdata = """--JfISa01
Content-Disposition: form-data; name="submit-name"
Content-Length: 5

Larry
--JfISa01"""
    env = {
        'REQUEST_METHOD': 'POST',
        'CONTENT_TYPE': f'multipart/form-data; boundary={boundary}',
        'CONTENT_LENGTH': str(len(postdata))}
    fp = BytesIO(postdata.encode('latin-1'))
    fs = FieldStorage(fp, environ=env, encoding="latin-1")
    assert len(fs.list) == 1
    assert fs.list[0].name == 'submit-name'
    assert fs.list[0].value == 'Larry'


def test_field_storage_multipart_no_content_length():
    fp = BytesIO(b"""--MyBoundary
Content-Disposition: form-data; name="my-arg"; filename="foo"

Test

--MyBoundary--
""")
    env = {
        "REQUEST_METHOD": "POST",
        "CONTENT_TYPE": "multipart/form-data; boundary=MyBoundary",
        "wsgi.input": fp,
    }
    fields = FieldStorage(fp, environ=env)

    assert len(fields["my-arg"].file.read()) == 5


def test_fieldstorage_as_context_manager():
    fp = BytesIO(b'x' * 10)
    env = {'REQUEST_METHOD': 'PUT'}
    with FieldStorage(fp=fp, environ=env) as fs:
        content = fs.file.read()
        assert fs.file.closed is False
    assert fs.file.closed is True
    assert content == 'x' * 10
    with pytest.raises(ValueError, match='I/O operation on closed file'):
        fs.file.read()


_qs_result = {
    'key1': 'value1',
    'key2': ['value2x', 'value2y'],
    'key3': 'value3',
    'key4': 'value4'
}


def test_qs_and_url_encode():
    data = "key2=value2x&key3=value3&key4=value4"
    environ = {
        'CONTENT_LENGTH':   str(len(data)),
        'CONTENT_TYPE':     'application/x-www-form-urlencoded',
        'QUERY_STRING':     'key1=value1&key2=value2y',
        'REQUEST_METHOD':   'POST',
    }
    assert gen_result(data, environ) == _qs_result


def test_max_num_fields():
    # For application/x-www-form-urlencoded
    data = '&'.join(['a=a']*11)
    environ = {
        'CONTENT_LENGTH': str(len(data)),
        'CONTENT_TYPE': 'application/x-www-form-urlencoded',
        'REQUEST_METHOD': 'POST',
    }

    with pytest.raises(ValueError):
        FieldStorage(
            fp=BytesIO(data.encode()),
            environ=environ,
            max_num_fields=10,
        )

    # For multipart/form-data
    data = """---123
Content-Disposition: form-data; name="a"

3
---123
Content-Type: application/x-www-form-urlencoded

a=4
---123
Content-Type: application/x-www-form-urlencoded

a=5
---123--
"""
    environ = {
        'CONTENT_LENGTH':   str(len(data)),
        'CONTENT_TYPE':     'multipart/form-data; boundary=-123',
        'QUERY_STRING':     'a=1&a=2',
        'REQUEST_METHOD':   'POST',
    }

    # 2 GET entities
    # 1 top level POST entities
    # 1 entity within the second POST entity
    # 1 entity within the third POST entity
    with pytest.raises(ValueError):
        FieldStorage(
            fp=BytesIO(data.encode()),
            environ=environ,
            max_num_fields=4,
        )
    FieldStorage(
        fp=BytesIO(data.encode()),
        environ=environ,
        max_num_fields=5,
    )


def test_qs_and_form_data():
    data = """---123
Content-Disposition: form-data; name="key2"

value2y
---123
Content-Disposition: form-data; name="key3"

value3
---123
Content-Disposition: form-data; name="key4"

value4
---123--
"""
    environ = {
        'CONTENT_LENGTH':   str(len(data)),
        'CONTENT_TYPE':     'multipart/form-data; boundary=-123',
        'QUERY_STRING':     'key1=value1&key2=value2x',
        'REQUEST_METHOD':   'POST',
    }
    assert gen_result(data, environ) == _qs_result


def test_qs_and_form_data_file():
    data = """---123
Content-Disposition: form-data; name="key2"

value2y
---123
Content-Disposition: form-data; name="key3"

value3
---123
Content-Disposition: form-data; name="key4"

value4
---123
Content-Disposition: form-data; name="upload"; filename="fake.txt"
Content-Type: text/plain

this is the content of the fake file

---123--
"""
    environ = {
        'CONTENT_LENGTH':   str(len(data)),
        'CONTENT_TYPE':     'multipart/form-data; boundary=-123',
        'QUERY_STRING':     'key1=value1&key2=value2x',
        'REQUEST_METHOD':   'POST',
    }
    result = {**_qs_result,
              'upload': b'this is the content of the fake file\n'}
    assert gen_result(data, environ) == result


def test_parse_header():
    assert parse_header("text/plain") == ("text/plain", {})
    assert parse_header("text/vnd.just.made.this.up ; ") == (
        "text/vnd.just.made.this.up", {})
    assert parse_header("text/plain;charset=us-ascii") == (
        "text/plain", {"charset": "us-ascii"})
    assert parse_header('text/plain ; charset="us-ascii"') == (
        "text/plain", {"charset": "us-ascii"})
    assert parse_header('text/plain ; charset="us-ascii"; another=opt') == (
        "text/plain", {"charset": "us-ascii", "another": "opt"})
    assert parse_header('attachment; filename="silly.txt"') == (
        "attachment", {"filename": "silly.txt"})
    assert parse_header('attachment; filename="strange;name"') == (
        "attachment", {"filename": "strange;name"})
    assert parse_header('attachment; filename="strange;name";size=123;') == (
        "attachment", {"filename": "strange;name", "size": "123"})
    assert parse_header('form-data; name="files"; filename="fo\\"o;bar"') == (
        "form-data", {"name": "files", "filename": 'fo"o;bar'})


BOUNDARY = "---------------------------721837373350705526688164684"
POSTDATA = """-----------------------------721837373350705526688164684
Content-Disposition: form-data; name="id"

1234
-----------------------------721837373350705526688164684
Content-Disposition: form-data; name="title"


-----------------------------721837373350705526688164684
Content-Disposition: form-data; name="file"; filename="test.txt"
Content-Type: text/plain

Testing 123.

-----------------------------721837373350705526688164684
Content-Disposition: form-data; name="submit"

 Add\x20
-----------------------------721837373350705526688164684--
"""

POSTDATA_NON_ASCII = """-----------------------------721837373350705526688164684
Content-Disposition: form-data; name="id"

\xe7\xf1\x80
-----------------------------721837373350705526688164684
"""

# http://www.w3.org/TR/html401/interact/forms.html#h-17.13.4
BOUNDARY_W3 = "AaB03x"
POSTDATA_W3 = """--AaB03x
Content-Disposition: form-data; name="submit-name"

Larry
--AaB03x
Content-Disposition: form-data; name="files"
Content-Type: multipart/mixed; boundary=BbC04y

--BbC04y
Content-Disposition: file; filename="file1.txt"
Content-Type: text/plain

... contents of file1.txt ...
--BbC04y
Content-Disposition: file; filename="file2.gif"
Content-Type: image/gif
Content-Transfer-Encoding: binary

...contents of file2.gif...
--BbC04y--
--AaB03x--
"""



from paste.util import quoting
import unittest

class TestQuoting(unittest.TestCase):
    def test_html_unquote(self):
        self.assertEqual(quoting.html_unquote(b'&lt;hey&nbsp;you&gt;'),
                         '<hey\xa0you>')
        self.assertEqual(quoting.html_unquote(b''),
                         '')
        self.assertEqual(quoting.html_unquote(b'&blahblah;'),
                         '&blahblah;')
        self.assertEqual(quoting.html_unquote(b'\xe1\x80\xa9'),
                         '\u1029')

    def test_html_quote(self):
        self.assertEqual(quoting.html_quote(1),
                         '1')
        self.assertEqual(quoting.html_quote(None),
                         '')
        self.assertEqual(quoting.html_quote('<hey!>'),
                         '&lt;hey!&gt;')
        self.assertEqual(quoting.html_quote(b'<hey!>'),
                         b'&lt;hey!&gt;')
        self.assertEqual(quoting.html_quote('<\u1029>'),
                         '&lt;\u1029&gt;')



import pytest
from paste.fixture import TestApp
from paste.recursive import RecursiveMiddleware, ForwardRequestException
from .test_errordocument import simple_app

def error_docs_app(environ, start_response):
    if environ['PATH_INFO'] == '/not_found':
        start_response("404 Not found", [('Content-type', 'text/plain')])
        return [b'Not found']
    elif environ['PATH_INFO'] == '/error':
        start_response("200 OK", [('Content-type', 'text/plain')])
        return [b'Page not found']
    elif environ['PATH_INFO'] == '/recurse':
        raise ForwardRequestException('/recurse')
    else:
        return simple_app(environ, start_response)

class Middleware:
    def __init__(self, app, url='/error'):
        self.app = app
        self.url = url
    def __call__(self, environ, start_response):
        raise ForwardRequestException(self.url)

def forward(app):
    app = TestApp(RecursiveMiddleware(app))
    res = app.get('')
    assert res.header('content-type') == 'text/plain'
    assert res.full_status == '200 OK'
    assert 'requested page returned' in res
    res = app.get('/error')
    assert res.header('content-type') == 'text/plain'
    assert res.full_status == '200 OK'
    assert 'Page not found' in res
    res = app.get('/not_found')
    assert res.header('content-type') == 'text/plain'
    assert res.full_status == '200 OK'
    assert 'Page not found' in res
    try:
        res = app.get('/recurse')
    except AssertionError as e:
        if str(e).startswith('Forwarding loop detected'):
            pass
        else:
            raise AssertionError('Failed to detect forwarding loop')

def test_ForwardRequest_url():
    class TestForwardRequestMiddleware(Middleware):
        def __call__(self, environ, start_response):
            if environ['PATH_INFO'] != '/not_found':
                return self.app(environ, start_response)
            raise ForwardRequestException(self.url)
    forward(TestForwardRequestMiddleware(error_docs_app))

def test_ForwardRequest_environ():
    class TestForwardRequestMiddleware(Middleware):
        def __call__(self, environ, start_response):
            if environ['PATH_INFO'] != '/not_found':
                return self.app(environ, start_response)
            environ['PATH_INFO'] = self.url
            raise ForwardRequestException(environ=environ)
    forward(TestForwardRequestMiddleware(error_docs_app))

def test_ForwardRequest_factory():

    from paste.errordocument import StatusKeeper

    class TestForwardRequestMiddleware(Middleware):
        def __call__(self, environ, start_response):
            if environ['PATH_INFO'] != '/not_found':
                return self.app(environ, start_response)
            environ['PATH_INFO'] = self.url
            def factory(app):
                return StatusKeeper(app, status='404 Not Found', url='/error', headers=[])
            raise ForwardRequestException(factory=factory)

    app = TestForwardRequestMiddleware(error_docs_app)
    app = TestApp(RecursiveMiddleware(app))
    res = app.get('')
    assert res.header('content-type') == 'text/plain'
    assert res.full_status == '200 OK'
    assert 'requested page returned' in res
    res = app.get('/error')
    assert res.header('content-type') == 'text/plain'
    assert res.full_status == '200 OK'
    assert 'Page not found' in res
    res = app.get('/not_found', status=404)
    assert res.header('content-type') == 'text/plain'
    assert res.full_status == '404 Not Found' # Different status
    assert 'Page not found' in res
    try:
        res = app.get('/recurse')
    except AssertionError as e:
        if str(e).startswith('Forwarding loop detected'):
            pass
        else:
            raise AssertionError('Failed to detect forwarding loop')

# Test Deprecated Code
def test_ForwardRequestException():
    class TestForwardRequestExceptionMiddleware(Middleware):
        def __call__(self, environ, start_response):
            if environ['PATH_INFO'] != '/not_found':
                return self.app(environ, start_response)
            raise ForwardRequestException(path_info=self.url)

    with pytest.warns(DeprecationWarning):
        forward(TestForwardRequestExceptionMiddleware(error_docs_app))



from paste.fixture import TestApp
from paste.exceptions.errormiddleware import ErrorMiddleware
from paste import lint
from paste.util.quoting import strip_html
#
# For some strange reason, these 4 lines cannot be removed or the regression
# test breaks; is it counting the number of lines in the file somehow?
#

def do_request(app, expect_status=500):
    app = lint.middleware(app)
    app = ErrorMiddleware(app, {}, debug=True)
    app = clear_middleware(app)
    testapp = TestApp(app)
    res = testapp.get('', status=expect_status,
                      expect_errors=True)
    return res

def clear_middleware(app):
    """
    The fixture sets paste.throw_errors, which suppresses exactly what
    we want to test in this case. This wrapper also strips exc_info
    on the *first* call to start_response (but not the second, or
    subsequent calls.
    """
    def clear_throw_errors(environ, start_response):
        headers_sent = []
        def replacement(status, headers, exc_info=None):
            if headers_sent:
                return start_response(status, headers, exc_info)
            headers_sent.append(True)
            return start_response(status, headers)
        if 'paste.throw_errors' in environ:
            del environ['paste.throw_errors']
        return app(environ, replacement)
    return clear_throw_errors


############################################################
## Applications that raise exceptions
############################################################

def bad_app():
    "No argument list!"
    return None

def unicode_bad_app(environ, start_response):
    raise ValueError(u"\u1000")

def start_response_app(environ, start_response):
    "raise error before start_response"
    raise ValueError("hi")

def after_start_response_app(environ, start_response):
    start_response("200 OK", [('Content-type', 'text/plain')])
    raise ValueError('error2')

def iter_app(environ, start_response):
    start_response("200 OK", [('Content-type', 'text/plain')])
    return yielder([b'this', b' is ', b' a', None])

def yielder(args):
    for arg in args:
        if arg is None:
            raise ValueError("None raises error")
        yield arg

############################################################
## Tests
############################################################

def test_makes_exception():
    res = do_request(bad_app)
    assert '<html' in res
    res = strip_html(str(res))
    assert 'bad_app() takes 0 positional arguments but 2 were given' in res
    assert 'iterator = application(environ, start_response_wrapper)' in res
    assert 'paste.lint' in res
    assert 'paste.exceptions.errormiddleware' in res

def test_unicode_exception():
    res = do_request(unicode_bad_app)


def test_start_res():
    res = do_request(start_response_app)
    res = strip_html(str(res))
    assert 'ValueError: hi' in res
    assert 'test_error_middleware' in res
    assert ':52 in start_response_app' in res

def test_after_start():
    res = do_request(after_start_response_app, 200)
    res = strip_html(str(res))
    #print res
    assert 'ValueError: error2' in res

def test_iter_app():
    res = do_request(lint.middleware(iter_app), 200)
    #print res
    assert 'None raises error' in res
    assert 'yielder' in res







# (c) 2007 Philip Jenvey; written for Paste (http://pythonpaste.org)
# Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php

import pytest

from paste.config import CONFIG, ConfigMiddleware
from paste.fixture import TestApp

test_key = 'test key'

def reset_config():
    while True:
        try:
            CONFIG._pop_object()
        except IndexError:
            break

def app_with_config(environ, start_response):
    start_response('200 OK', [('Content-type','text/plain')])
    lines = ['Variable is: %s\n' % CONFIG[test_key],
            'Variable is (in environ): %s' % environ['paste.config'][test_key]]
    lines = [line.encode('utf8') for line in lines]
    return lines

class NestingAppWithConfig:
    def __init__(self, app):
        self.app = app

    def __call__(self, environ, start_response):
        response = self.app(environ, start_response)
        assert isinstance(response, list)
        supplement = ['Nesting variable is: %s' % CONFIG[test_key],
                      'Nesting variable is (in environ): %s' % \
                          environ['paste.config'][test_key]]
        supplement = [line.encode('utf8') for line in supplement]
        response.extend(supplement)
        return response

def test_request_config():
    try:
        config = {test_key: 'test value'}
        app = ConfigMiddleware(app_with_config, config)
        res = TestApp(app).get('/')
        assert 'Variable is: test value' in res
        assert 'Variable is (in environ): test value' in res
    finally:
        reset_config()

def test_request_config_multi():
    try:
        config = {test_key: 'test value'}
        app = ConfigMiddleware(app_with_config, config)
        config = {test_key: 'nesting value'}
        app = ConfigMiddleware(NestingAppWithConfig(app), config)
        res = TestApp(app).get('/')
        assert 'Variable is: test value' in res
        assert 'Variable is (in environ): test value' in res
        assert 'Nesting variable is: nesting value' in res
        print(res)
        assert 'Nesting variable is (in environ): nesting value' in res
    finally:
        reset_config()

def test_process_config(request_app=test_request_config):
    try:
        process_config = {test_key: 'bar', 'process_var': 'foo'}
        CONFIG.push_process_config(process_config)

        assert CONFIG[test_key] == 'bar'
        assert CONFIG['process_var'] == 'foo'

        request_app()

        assert CONFIG[test_key] == 'bar'
        assert CONFIG['process_var'] == 'foo'
        CONFIG.pop_process_config()

        pytest.raises(AttributeError, lambda: 'process_var' not in CONFIG)
        pytest.raises(IndexError, CONFIG.pop_process_config)
    finally:
        reset_config()

def test_process_config_multi():
    test_process_config(test_request_config_multi)



import pytest

from paste.wsgilib import add_close


def app_iterable_func_bytes():
    yield b'a'
    yield b'b'
    yield b'c'


def app_iterable_func_unicode():
    yield b'a'.decode('ascii')
    yield b'b'.decode('ascii')
    yield b'c'.decode('ascii')


def close_func():
    global close_func_called
    close_func_called = True


@pytest.mark.parametrize("app_iterable_func,expected", [
    (app_iterable_func_bytes, [b'a', b'b', b'c']),
    (app_iterable_func_unicode, ['a', 'b', 'c']),
])
def test_add_close(app_iterable_func, expected):
    global close_func_called

    close_func_called = False
    lst = []
    app_iterable = app_iterable_func()

    obj = add_close(app_iterable, close_func)
    for x in obj:
        lst.append(x)
    obj.close()

    assert lst == expected
    assert close_func_called
    assert obj._closed



# (c) 2005 Ian Bicking and contributors; written for Paste (http://pythonpaste.org)
# Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php

import warnings

try:
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", category=DeprecationWarning)
        import pkg_resources
        pkg_resources.declare_namespace(__name__)
except (AttributeError, ImportError):
    # don't prevent use of paste if pkg_resources isn't installed
    from pkgutil import extend_path
    __path__ = extend_path(__path__, __name__)

try:
    import modulefinder
except ImportError:
    pass
else:
    for p in __path__:
        modulefinder.AddPackagePath(__name__, p)



from paste.response import replace_header

def test_replace_header():
    h = [('content-type', 'text/plain'),
         ('x-blah', 'foobar')]
    replace_header(h, 'content-length', '10')
    assert h[-1] == ('content-length', '10')
    replace_header(h, 'Content-Type', 'text/html')
    assert ('content-type', 'text/html') in h
    assert ('content-type', 'text/plain') not in h




import time

from paste.httpheaders import (
    normalize_headers,
    ACCEPT,
    ACCEPT_CHARSET,
    AGE,
    ALLOW,
    CACHE_CONTROL,
    CONTENT_DISPOSITION,
    CONTENT_ENCODING,
    CONTENT_LENGTH,
    CONTENT_TYPE,
    DATE,
    EXPIRES,
    ETAG,
    FROM,
    IF_MODIFIED_SINCE,
    PRAGMA,
    RANGE,
    REFERER,
    UPGRADE,
    VIA,
    VARY,
    WWW_AUTHENTICATE,
)

def _test_generic(collection):
    assert 'bing' == VIA(collection)
    REFERER.update(collection,'internal:/some/path')
    assert 'internal:/some/path' == REFERER(collection)
    CACHE_CONTROL.update(collection,max_age=1234)
    CONTENT_DISPOSITION.update(collection,filename="bingles.txt")
    PRAGMA.update(collection,"test","multi",'valued="items"')
    assert 'public, max-age=1234' == CACHE_CONTROL(collection)
    assert 'attachment; filename="bingles.txt"' == \
            CONTENT_DISPOSITION(collection)
    assert 'test, multi, valued="items"' == PRAGMA(collection)
    VIA.delete(collection)


def test_environ():
    collection = {'HTTP_VIA':'bing', 'wsgi.version': '1.0' }
    _test_generic(collection)
    assert collection == {'wsgi.version': '1.0',
      'HTTP_PRAGMA': 'test, multi, valued="items"',
      'HTTP_REFERER': 'internal:/some/path',
      'HTTP_CONTENT_DISPOSITION': 'attachment; filename="bingles.txt"',
      'HTTP_CACHE_CONTROL': 'public, max-age=1234'
    }

def test_environ_cgi():
    environ = {'CONTENT_TYPE': 'text/plain', 'wsgi.version': '1.0',
               'HTTP_CONTENT_TYPE': 'ignored/invalid',
               'CONTENT_LENGTH': '200'}
    assert 'text/plain' == CONTENT_TYPE(environ)
    assert '200' == CONTENT_LENGTH(environ)
    CONTENT_TYPE.update(environ,'new/type')
    assert 'new/type' == CONTENT_TYPE(environ)
    CONTENT_TYPE.delete(environ)
    assert '' == CONTENT_TYPE(environ)
    assert 'ignored/invalid' == environ['HTTP_CONTENT_TYPE']

def test_response_headers():
    collection = [('via', 'bing')]
    _test_generic(collection)
    normalize_headers(collection)
    assert collection == [
        ('Cache-Control', 'public, max-age=1234'),
        ('Pragma', 'test, multi, valued="items"'),
        ('Referer', 'internal:/some/path'),
        ('Content-Disposition', 'attachment; filename="bingles.txt"')
    ]

def test_cache_control():
    assert 'public' == CACHE_CONTROL()
    assert 'public' == CACHE_CONTROL(public=True)
    assert 'private' == CACHE_CONTROL(private=True)
    assert 'no-cache' == CACHE_CONTROL(no_cache=True)
    assert 'private, no-store' == CACHE_CONTROL(private=True, no_store=True)
    assert 'public, max-age=60' == CACHE_CONTROL(max_age=60)
    assert 'public, max-age=86400' == \
            CACHE_CONTROL(max_age=CACHE_CONTROL.ONE_DAY)
    CACHE_CONTROL.extensions['community'] = str
    assert 'public, community="bingles"' == \
            CACHE_CONTROL(community="bingles")
    headers = []
    CACHE_CONTROL.apply(headers,max_age=60)
    assert 'public, max-age=60' == CACHE_CONTROL(headers)
    assert EXPIRES.parse(headers) > time.time()
    assert EXPIRES.parse(headers) < time.time() + 60

def test_content_disposition():
    assert 'attachment' == CONTENT_DISPOSITION()
    assert 'attachment' == CONTENT_DISPOSITION(attachment=True)
    assert 'inline' == CONTENT_DISPOSITION(inline=True)
    assert 'inline; filename="test.txt"' == \
            CONTENT_DISPOSITION(inline=True, filename="test.txt")
    assert 'attachment; filename="test.txt"' == \
            CONTENT_DISPOSITION(filename="/some/path/test.txt")
    headers = []
    CONTENT_DISPOSITION.apply(headers,filename="test.txt")
    assert 'text/plain' == CONTENT_TYPE(headers)
    CONTENT_DISPOSITION.apply(headers,filename="test")
    assert 'text/plain' == CONTENT_TYPE(headers)
    CONTENT_DISPOSITION.apply(headers,filename="test.html")
    assert 'text/plain' == CONTENT_TYPE(headers)
    headers = [('Content-Type', 'application/octet-stream')]
    CONTENT_DISPOSITION.apply(headers,filename="test.txt")
    assert 'text/plain' == CONTENT_TYPE(headers)
    assert headers == [
      ('Content-Type', 'text/plain'),
      ('Content-Disposition', 'attachment; filename="test.txt"')
    ]

def test_range():
    assert ('bytes',[(0,300)]) == RANGE.parse("bytes=0-300")
    assert ('bytes',[(0,300)]) == RANGE.parse("bytes   =  -300")
    assert ('bytes',[(0,None)]) == RANGE.parse("bytes=  -")
    assert ('bytes',[(0,None)]) == RANGE.parse("bytes=0   -   ")
    assert ('bytes',[(300,None)]) == RANGE.parse("   BYTES=300-")
    assert ('bytes',[(4,5),(6,7)]) == RANGE.parse(" Bytes = 4 - 5,6 - 07  ")
    assert ('bytes',[(0,5),(7,None)]) == RANGE.parse(" bytes=-5,7-")
    assert ('bytes',[(0,5),(7,None)]) == RANGE.parse(" bytes=-5,7-")
    assert ('bytes',[(0,5),(7,None)]) == RANGE.parse(" bytes=-5,7-")
    assert None == RANGE.parse("")
    assert None == RANGE.parse("bytes=0,300")
    assert None == RANGE.parse("bytes=-7,5-")

def test_copy():
    environ = {'HTTP_VIA':'bing', 'wsgi.version': '1.0' }
    response_headers = []
    VIA.update(response_headers,environ)
    assert response_headers == [('Via', 'bing')]

def test_sorting():
    # verify the HTTP_HEADERS are set with their canonical form
    sample = [WWW_AUTHENTICATE, VIA, ACCEPT, DATE,
    ACCEPT_CHARSET, AGE, ALLOW, CACHE_CONTROL,
    CONTENT_ENCODING, ETAG, CONTENT_TYPE, FROM,
    EXPIRES, RANGE, UPGRADE, VARY, ALLOW]
    sample.sort()
    sample = [str(x) for x in sample]
    assert sample == [
     # general headers first
     'Cache-Control', 'Date', 'Upgrade', 'Via',
     # request headers next
     'Accept', 'Accept-Charset', 'From', 'Range',
     # response headers following
     'Age', 'ETag', 'Vary', 'WWW-Authenticate',
     # entity headers (/w expected duplicate)
     'Allow', 'Allow', 'Content-Encoding', 'Content-Type', 'Expires'
    ]

def test_normalize():
    response_headers = [
       ('www-authenticate','Response AuthMessage'),
       ('unknown-header','Unknown Sorted Last'),
       ('Via','General Bingles'),
       ('aLLoW','Entity Allow Something'),
       ('ETAG','Response 34234'),
       ('expires','Entity An-Expiration-Date'),
       ('date','General A-Date')]
    normalize_headers(response_headers, strict=False)
    assert response_headers == [
     ('Date', 'General A-Date'),
     ('Via', 'General Bingles'),
     ('ETag', 'Response 34234'),
     ('WWW-Authenticate', 'Response AuthMessage'),
     ('Allow', 'Entity Allow Something'),
     ('Expires', 'Entity An-Expiration-Date'),
     ('Unknown-Header', 'Unknown Sorted Last')]

def test_if_modified_since():
    from paste.httpexceptions import HTTPBadRequest
    date = 'Thu, 34 Jul 3119 29:34:18 GMT'
    try:
        x = IF_MODIFIED_SINCE.parse({'HTTP_IF_MODIFIED_SINCE': date,
                                     'wsgi.version': (1, 0)})
    except HTTPBadRequest:
        pass
    else:
        assert 0



from paste.fixture import TestApp
from paste.cgitb_catcher import CgitbMiddleware
from paste import lint
from .test_exceptions.test_error_middleware import clear_middleware

def do_request(app, expect_status=500):
    app = lint.middleware(app)
    app = CgitbMiddleware(app, {}, display=True)
    app = clear_middleware(app)
    testapp = TestApp(app)
    res = testapp.get('', status=expect_status,
                      expect_errors=True)
    return res


############################################################
## Applications that raise exceptions
############################################################

def bad_app():
    "No argument list!"
    return None

def start_response_app(environ, start_response):
    "raise error before start_response"
    raise ValueError("hi")

def after_start_response_app(environ, start_response):
    start_response("200 OK", [('Content-type', 'text/plain')])
    raise ValueError('error2')

def iter_app(environ, start_response):
    start_response("200 OK", [('Content-type', 'text/plain')])
    return yielder([b'this', b' is ', b' a', None])

def yielder(args):
    for arg in args:
        if arg is None:
            raise ValueError("None raises error")
        yield arg

############################################################
## Tests
############################################################

def test_makes_exception():
    res = do_request(bad_app)
    print(res)
    assert 'bad_app() takes 0 positional arguments but 2 were given' in res
    assert 'iterator = application(environ, start_response_wrapper)' in res
    assert 'lint.py' in res
    assert 'cgitb_catcher.py' in res

def test_start_res():
    res = do_request(start_response_app)
    print(res)
    assert 'ValueError: hi' in res
    assert 'test_cgitb_catcher.py' in res
    assert 'line 26, in start_response_app' in res

def test_after_start():
    res = do_request(after_start_response_app, 200)
    print(res)
    assert 'ValueError: error2' in res
    assert 'line 30' in res

def test_iter_app():
    res = do_request(iter_app, 200)
    print(res)
    assert 'None raises error' in res
    assert 'yielder' in res







import pytest

from paste import proxy
from paste.fixture import TestApp

# TODO: Skipping this for now as it is unreliable. Ideally we'd run something
# locally and not have to rely on external stuff.
@pytest.mark.skip(reason="httpbin.org is too slow these days")
def test_proxy_to_website():
    # Not the most robust test...
    # need to test things like POSTing to pages, and getting from pages
    # that don't set content-length.
    app = proxy.Proxy('http://httpbin.org')
    app = TestApp(app)
    res = app.get('/')
    # httpbin is a react app now, so hard to read
    assert '<title>httpbin.org</title>' in res



# Slightly modified tests from the original cgitb test module.

# Copyright © 2001-2023 Python Software Foundation; All Rights Reserved.

import sys

import pytest

from paste.util.cgitb_hook import small, strong, grey, html, text, Hook


def test_fonts():
    text = "Hello Robbie!"
    assert small(text) == f'<small>{text}</small>'
    assert strong(text) == f'<strong>{text}</strong>'
    assert grey(text) == f'<font color="#909090">{text}</font>'


def test_blanks():
    assert small('') == ''
    assert strong('') == ''
    assert grey('') == ''


def test_html():
    try:
        raise ValueError("Hello World")
    except ValueError as err:
        # If the html was templated we could do a bit more here.
        # At least check that we get details on what we just raised.
        out = html(sys.exc_info())
        assert 'ValueError' in out
        assert str(err) in out


def test_text():
    try:
        raise ValueError("Hello World")
    except ValueError:
        out = text(sys.exc_info())
        assert 'ValueError' in out
        assert 'Hello World' in out


def dummy_error():
    raise RuntimeError("Hello World")


@pytest.mark.parametrize('format', (None, 'html', 'text'))
def test_syshook_no_logdir_default_format(format, capsys, tmp_path):
    excepthook = sys.excepthook
    args = {'logdir': tmp_path}
    if format:
        args['format'] = format
    hook = Hook(**args)
    try:
        dummy_error()
    except RuntimeError as err:
        hook(err.__class__, err, err.__traceback__)
    finally:
        sys.excepthook = excepthook

    log_files = list(tmp_path.glob('*.txt' if format == 'text' else '*.html'))
    assert len(log_files) == 1
    log_file = log_files[0]
    out = log_file.open('r').read()
    log_file.unlink()

    assert 'A problem occurred in a Python script.' in out
    assert 'RuntimeError' in out
    assert 'Hello World' in out
    assert 'test_syshook_no_logdir_default_format' in out
    if format == 'text':
        assert 'in dummy_error' in out
    else:
        assert 'in <strong>dummy_error</strong>' in out
        assert '<p>' in out
        assert '</p>' in out

    assert 'Content-Type: text/html' not in out
    assert 'contains the description of this error' not in out

    captured = capsys.readouterr()
    assert not captured.err
    output = captured.out

    if format != 'text':
        assert 'Content-Type: text/html' in output
    assert out in output
    assert log_file.name in output
    assert 'contains the description of this error' in output



import hashlib
import base64
from http.cookies import SimpleCookie
from paste.auth.auth_tkt import AuthTicket


def test_auth_ticket_digest_and_cookie_value():
    test_parameters = [
        (
            (
                'shared_secret',
                'username',
                '0.0.0.0',  # remote address
            ),
            {
                'tokens': ['admin'],
                'time': 1579782607
            },
            b'731274bec45f6983c1f33bac8e8baf43',
            b'731274bec45f6983c1f33bac8e8baf435e2991cfusername!admin!',
        ),
        (
            (
                'shared_secret',
                'username',
                '0.0.0.0',
            ),
            {
                'tokens': ['admin'],
                'time': 1579782607,
                'digest_algo': hashlib.sha512
            },
            b'09e72a63c57ca4cfeca5fa578646deb2b27f7a461d91ad9aa32b85c93ef6fa7744ac006eb3d9a71a36375b5ab50cbae072bb3042e2a59198b7f314900cba4423',
            b'09e72a63c57ca4cfeca5fa578646deb2b27f7a461d91ad9aa32b85c93ef6fa7744ac006eb3d9a71a36375b5ab50cbae072bb3042e2a59198b7f314900cba44235e2991cfusername!admin!',
        ),
    ]

    for test_args, test_kwargs, expected_digest, expected_cookie_value in test_parameters:
        token = AuthTicket(*test_args, **test_kwargs)
        assert expected_digest == token.digest()
        assert expected_cookie_value == token.cookie_value()


def test_auth_ticket_cookie():
    test_parameters = [
        (
            (
                'shared_secret',
                'username',
                '0.0.0.0',  # remote address
            ),
            {
                'tokens': ['admin'],
                'time': 1579782607
            },
            {
                'name': 'auth_tkt',
                'path': '/',
                'secure': '',
                'cookie_value': b'731274bec45f6983c1f33bac8e8baf435e2991cfusername!admin!'
            }
        ),
        (
            (
                'shared_secret',
                'username',
                '0.0.0.0',  # remote address
            ),
            {
                'tokens': ['admin'],
                'time': 1579782607,
                'secure': True
            },
            {
                'name': 'auth_tkt',
                'path': '/',
                'secure': 'true',
                'cookie_value': b'731274bec45f6983c1f33bac8e8baf435e2991cfusername!admin!'
            }
        ),
        (
            (
                'shared_secret',
                'username',
                '0.0.0.0',  # remote address
            ),
            {
                'tokens': ['admin'],
                'time': 1579782607,
                'cookie_name': 'custom_cookie',
                'secure': False
            },
            {
                'name': 'custom_cookie',
                'path': '/',
                'secure': '',
                'cookie_value': b'731274bec45f6983c1f33bac8e8baf435e2991cfusername!admin!'
            }
        ),
    ]

    for test_args, test_kwargs, expected_values in test_parameters:
        token = AuthTicket(*test_args, **test_kwargs)
        expected_cookie = SimpleCookie()
        # import pdb; pdb.set_trace()
        expected_cookie_value = base64.b64encode(expected_values['cookie_value'])

        expected_cookie[expected_values['name']] = expected_cookie_value
        expected_cookie[expected_values['name']]['path'] = expected_values['path']
        expected_cookie[expected_values['name']]['secure'] = expected_values['secure']
        assert expected_cookie == token.cookie()



# (c) 2005 Clark C. Evans
# This module is part of the Python Paste Project and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from paste.auth.digest import digest_password, AuthDigestHandler
from paste.wsgilib import raw_interactive
from paste.httpexceptions import HTTPExceptionHandler
from paste.httpheaders import AUTHORIZATION, WWW_AUTHENTICATE, REMOTE_USER
import os

def application(environ, start_response):
    content = REMOTE_USER(environ)
    start_response("200 OK",(('Content-Type', 'text/plain'),
                             ('Content-Length', len(content))))

    content = content.encode('utf8')
    return [content]

realm = "tag:clarkevans.com,2005:testing"

def backwords(environ, realm, username):
    """ dummy password hash, where user password is just reverse """
    password = list(username)
    password.reverse()
    password = "".join(password)
    return digest_password(realm, username, password)

application = AuthDigestHandler(application,realm,backwords)
application = HTTPExceptionHandler(application)

def check(username, password, path="/"):
    """ perform two-stage authentication to verify login """
    (status,headers,content,errors) = \
        raw_interactive(application,path, accept='text/html')
    assert status.startswith("401")
    challenge = WWW_AUTHENTICATE(headers)
    response = AUTHORIZATION(username=username, password=password,
                             challenge=challenge, path=path)
    assert "Digest" in response and username in response
    (status,headers,content,errors) = \
        raw_interactive(application,path,
                        HTTP_AUTHORIZATION=response)
    if status.startswith("200"):
        return content
    if status.startswith("401"):
        return None
    assert False, "Unexpected Status: %s" % status

def test_digest():
    assert b'bing' == check("bing","gnib")
    assert check("bing","bad") is None

#
# The following code uses sockets to test the functionality,
# to enable use:
#
# $ TEST_SOCKET=1 pytest


if os.environ.get("TEST_SOCKET", ""):
    from urllib.error import HTTPError
    from urllib.request import build_opener, HTTPDigestAuthHandler
    from paste.debug.testserver import serve
    server = serve(application)

    def authfetch(username,password,path="/",realm=realm):
        server.accept(2)
        import socket
        socket.setdefaulttimeout(5)
        uri = ("http://%s:%s" % server.server_address) + path
        auth = HTTPDigestAuthHandler()
        auth.add_password(realm,uri,username,password)
        opener = build_opener(auth)
        result = opener.open(uri)
        return result.read()

    def test_success():
        assert "bing" == authfetch('bing','gnib')

    def test_failure():
        # urllib tries 5 more times before it gives up
        server.accept(5)
        try:
            authfetch('bing','wrong')
            assert False, "this should raise an exception"
        except HTTPError as e:
            assert e.code == 401

    def test_shutdown():
        server.stop()




import doctest
import os

import pytest

from paste.util.import_string import simple_import

filenames = [
    'tests/template.txt',
    ]

modules = [
    'paste.util.template',
    'paste.util.looper',
    # This one opens up httpserver, which is bad:
    #'paste.auth.cookie',
    #'paste.auth.multi',
    #'paste.auth.digest',
    #'paste.auth.basic',
    #'paste.auth.form',
    #'paste.progress',
    'paste.exceptions.serial_number_generator',
    'paste.evalexception.evalcontext',
    'paste.util.dateinterval',
    'paste.util.quoting',
    'paste.wsgilib',
    'paste.url',
    'paste.request',
    ]

options = doctest.ELLIPSIS | doctest.REPORT_ONLY_FIRST_FAILURE
options |= doctest.IGNORE_EXCEPTION_DETAIL


@pytest.mark.parametrize('filename', filenames)
def test_doctests(filename):
    filename = os.path.join(
        os.path.dirname(os.path.dirname(__file__)),
        filename)
    failure, total = doctest.testfile(
        filename, module_relative=False,
        optionflags=options)
    assert not failure, "Failure in %r" % filename


@pytest.mark.parametrize('module', modules)
def test_doctest_mods(module):
    module = simple_import(module)
    failure, total = doctest.testmod(
        module, optionflags=options)
    assert not failure, "Failure in %r" % module


if __name__ == '__main__':
    import sys
    import doctest
    args = sys.argv[1:]
    if not args:
        args = filenames
    for filename in args:
        doctest.testfile(filename, module_relative=False)



from paste.fixture import TestApp
from paste.gzipper import middleware
import gzip
import io

def simple_app(environ, start_response):
    start_response('200 OK',
                   [('content-type', 'text/plain'),
                    ('content-length', '0')])
    return [b'this is a test'] if environ['REQUEST_METHOD'] != 'HEAD' else []

wsgi_app = middleware(simple_app)
app = TestApp(wsgi_app)

def test_gzip():
    res = app.get(
        '/', extra_environ=dict(HTTP_ACCEPT_ENCODING='gzip'))
    assert int(res.header('content-length')) == len(res.body)
    assert res.body != b'this is a test'
    actual = gzip.GzipFile(fileobj=io.BytesIO(res.body)).read()
    assert actual == b'this is a test'

def test_gzip_head():
    res = app.head(
        '/', extra_environ=dict(HTTP_ACCEPT_ENCODING='gzip'))
    assert int(res.header('content-length')) == 0
    assert res.body == b''



import os
import sys

import pytest

from paste.cgiapp import CGIApplication, CGIError
from paste.fixture import TestApp

data_dir = os.path.join(os.path.dirname(__file__), 'cgiapp_data')

pytestmark = pytest.mark.skipif(
    sys.platform == 'win32' or sys.platform.startswith('java'),
    reason="CGI scripts can't work on Windows or Jython")


# Ensure the CGI scripts are called with the same python interpreter.
# Put a symlink to the interpreter executable into the path...
def setup_module():
    global oldpath, pyexelink
    oldpath = os.environ.get('PATH', None)
    os.environ['PATH'] = data_dir + os.path.pathsep + oldpath
    pyexelink = os.path.join(data_dir, "python")
    try:
        os.unlink(pyexelink)
    except OSError:
        pass
    os.symlink(sys.executable, pyexelink)

# ... and clean up again.
def teardown_module():
    global oldpath, pyexelink
    os.unlink(pyexelink)
    if oldpath is not None:
        os.environ['PATH'] = oldpath
    else:
        del os.environ['PATH']

def test_ok():
    app = TestApp(CGIApplication({}, script='ok.cgi', path=[data_dir]))
    res = app.get('')
    assert res.header('content-type') == 'text/html; charset=UTF-8'
    assert res.full_status == '200 Okay'
    assert 'This is the body' in res

def test_form():
    app = TestApp(CGIApplication({}, script='form.cgi', path=[data_dir]))
    res = app.post('', params={'name': b'joe'},
                   upload_files=[('up', 'file.txt', b'x'*10000)])
    assert 'file.txt' in res
    assert 'joe' in res
    assert 'x'*10000 in res

def test_error():
    app = TestApp(CGIApplication({}, script='error.cgi', path=[data_dir]))
    pytest.raises(CGIError, app.get, '', status=500)

def test_stderr():
    app = TestApp(CGIApplication({}, script='stderr.cgi', path=[data_dir]))
    res = app.get('', expect_errors=True)
    assert res.status == 500
    assert 'error' in res
    assert 'some data' in res.errors



import io

from paste.request import parse_formvars, parse_querystring
from paste.util.multidict import MultiDict

def test_parse_querystring():
    e = {'QUERY_STRING': 'a=1&b=2&c=3&b=4'}
    d = parse_querystring(e)
    assert d == [('a', '1'), ('b', '2'), ('c', '3'), ('b', '4')]
    assert e['paste.parsed_querystring'] == (
        (d, e['QUERY_STRING']))
    e = {'QUERY_STRING': 'a&b&c=&d=1'}
    d = parse_querystring(e)
    assert d == [('a', ''), ('b', ''), ('c', ''), ('d', '1')]

def make_post(body):
    e = {
        'CONTENT_TYPE': 'application/x-www-form-urlencoded',
        'CONTENT_LENGTH': str(len(body)),
        'REQUEST_METHOD': 'POST',
        'wsgi.input': io.BytesIO(body),
        }
    return e

def test_parse_formvars_nodup():
    """GH85: Test that parse_formvars can be called twice."""
    e = {'QUERY_STRING': 'a=1&b=2&c=3&b=4', 'wsgi.input': ""}
    d1 = parse_formvars(e)
    assert d1.getall("a") == ["1"]
    d2 = parse_formvars(e)
    assert d2.getall("a") == ["1"]

def test_parsevars():
    e = make_post(b'a=1&b=2&c=3&b=4')
    #cur_input = e['wsgi.input']
    d = parse_formvars(e)
    assert isinstance(d, MultiDict)
    assert d == MultiDict([('a', '1'), ('b', '2'), ('c', '3'), ('b', '4')])
    assert e['paste.parsed_formvars'] == (
        (d, e['wsgi.input']))
    # XXX: http://trac.pythonpaste.org/pythonpaste/ticket/125
    #assert e['wsgi.input'] is not cur_input
    #cur_input.seek(0)
    #assert e['wsgi.input'].read() == cur_input.read()



from paste.errordocument import forward
from paste.fixture import TestApp
from paste.recursive import RecursiveMiddleware

def simple_app(environ, start_response):
    start_response("200 OK", [('Content-type', 'text/plain')])
    return [b'requested page returned']

def not_found_app(environ, start_response):
    start_response("404 Not found", [('Content-type', 'text/plain')])
    return [b'requested page returned']

def test_ok():
    app = TestApp(simple_app)
    res = app.get('')
    assert res.header('content-type') == 'text/plain'
    assert res.full_status == '200 OK'
    assert 'requested page returned' in res

def error_docs_app(environ, start_response):
    if environ['PATH_INFO'] == '/not_found':
        start_response("404 Not found", [('Content-type', 'text/plain')])
        return [b'Not found']
    elif environ['PATH_INFO'] == '/error':
        start_response("200 OK", [('Content-type', 'text/plain')])
        return [b'Page not found']
    else:
        return simple_app(environ, start_response)

def test_error_docs_app():
    app = TestApp(error_docs_app)
    res = app.get('')
    assert res.header('content-type') == 'text/plain'
    assert res.full_status == '200 OK'
    assert 'requested page returned' in res
    res = app.get('/error')
    assert res.header('content-type') == 'text/plain'
    assert res.full_status == '200 OK'
    assert 'Page not found' in res
    res = app.get('/not_found', status=404)
    assert res.header('content-type') == 'text/plain'
    assert res.full_status == '404 Not found'
    assert 'Not found' in res

def test_forward():
    app = forward(error_docs_app, codes={404:'/error'})
    app = TestApp(RecursiveMiddleware(app))
    res = app.get('')
    assert res.header('content-type') == 'text/plain'
    assert res.full_status == '200 OK'
    assert 'requested page returned' in res
    res = app.get('/error')
    assert res.header('content-type') == 'text/plain'
    assert res.full_status == '200 OK'
    assert 'Page not found' in res
    res = app.get('/not_found', status=404)
    assert res.header('content-type') == 'text/plain'
    assert res.full_status == '404 Not found'
    # Note changed response
    assert 'Page not found' in res

def auth_required_app(environ, start_response):
    start_response('401 Unauthorized', [('content-type', 'text/plain'), ('www-authenticate', 'Basic realm="Foo"')])
    return ['Sign in!']

def auth_docs_app(environ, start_response):
    if environ['PATH_INFO'] == '/auth':
        return auth_required_app(environ, start_response)
    elif environ['PATH_INFO'] == '/auth_doc':
        start_response("200 OK", [('Content-type', 'text/html')])
        return [b'<html>Login!</html>']
    else:
        return simple_app(environ, start_response)

def test_auth_docs_app():
    wsgi_app = forward(auth_docs_app, codes={401: '/auth_doc'})
    app = TestApp(wsgi_app)
    res = app.get('/auth_doc')
    assert res.header('content-type') == 'text/html'
    res = app.get('/auth', status=401)
    assert res.header('content-type') == 'text/html'
    assert res.header('www-authenticate') == 'Basic realm="Foo"'
    assert res.body == b'<html>Login!</html>'

def test_bad_error():
    def app(environ, start_response):
        start_response('404 Not Found', [('content-type', 'text/plain')])
        return ['not found']
    app = forward(app, {404: '/404.html'})
    app = TestApp(app)
    resp = app.get('/test', expect_errors=True)
    print(resp)



# (c) 2005 Clark C. Evans and contributors
# This module is part of the Python Paste Project and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
# Some of this code was funded by: http://prometheusresearch.com
from time import localtime
from datetime import date
from paste.util.datetimeutil import (
    normalize_date,
    normalize_time,
    normalize_timedelta,
    parse_date,
)

def test_timedelta():
    assert('' == normalize_timedelta(""))
    assert('0.10' == normalize_timedelta("6m"))
    assert('0.50' == normalize_timedelta("30m"))
    assert('0.75' == normalize_timedelta("45m"))
    assert('1.00' == normalize_timedelta("60 min"))
    assert('1.50' == normalize_timedelta("90min"))
    assert('1.50' == normalize_timedelta("1.50"))
    assert('4.50' == normalize_timedelta("4 : 30"))
    assert('1.50' == normalize_timedelta("1h 30m"))
    assert('1.00' == normalize_timedelta("1"))
    assert('1.00' == normalize_timedelta("1 hour"))
    assert('8.00' == normalize_timedelta("480 mins"))
    assert('8.00' == normalize_timedelta("8h"))
    assert('0.50' == normalize_timedelta("0.5"))
    assert('0.10' == normalize_timedelta(".1"))
    assert('0.50' == normalize_timedelta(".50"))
    assert('0.75' == normalize_timedelta("0.75"))

def test_time():
    assert('03:00 PM' == normalize_time("3p", ampm=True))
    assert('03:00 AM' == normalize_time("300", ampm=True))
    assert('03:22 AM' == normalize_time("322", ampm=True))
    assert('01:22 PM' == normalize_time("1322", ampm=True))
    assert('01:00 PM' == normalize_time("13", ampm=True))
    assert('12:00 PM' == normalize_time("noon", ampm=True))
    assert("06:00 PM" == normalize_time("6", ampm=True))
    assert("01:00 PM" == normalize_time("1", ampm=True))
    assert("07:00 AM" == normalize_time("7", ampm=True))
    assert("01:00 PM" == normalize_time("1 pm", ampm=True))
    assert("03:30 PM" == normalize_time("3:30 pm", ampm=True))
    assert("03:30 PM" == normalize_time("3 30 pm", ampm=True))
    assert("03:30 PM" == normalize_time("3 30 P.M.", ampm=True))
    assert("12:00 PM" == normalize_time("0", ampm=True))
    assert("12:00 AM" == normalize_time("1200 AM", ampm=True))

def test_date():
    tm = localtime()
    yr = tm[0]
    mo = tm[1]
    assert(date(yr,4,11)  == parse_date("411"))
    assert(date(yr,4,11)  == parse_date("APR11"))
    assert(date(yr,4,11)  == parse_date("11APR"))
    assert(date(yr,4,11)  == parse_date("4 11"))
    assert(date(yr,4,11)  == parse_date("11 APR"))
    assert(date(yr,4,11)  == parse_date("APR 11"))
    assert(date(yr,mo,11) == parse_date("11"))
    assert(date(yr,4,1)   == parse_date("APR"))
    assert(date(yr,4,11)  == parse_date("4/11"))
    assert(date.today()   == parse_date("today"))
    assert(date.today()   == parse_date("now"))
    assert(None           == parse_date(""))
    assert(''             == normalize_date(None))

    assert('2001-02-03' == normalize_date("20010203"))
    assert('1999-04-11' == normalize_date("1999 4 11"))
    assert('1999-04-11' == normalize_date("1999 APR 11"))
    assert('1999-04-11' == normalize_date("APR 11 1999"))
    assert('1999-04-11' == normalize_date("11 APR 1999"))
    assert('1999-04-11' == normalize_date("4 11 1999"))
    assert('1999-04-01' == normalize_date("1999 APR"))
    assert('1999-04-01' == normalize_date("1999 4"))
    assert('1999-04-01' == normalize_date("4 1999"))
    assert('1999-04-01' == normalize_date("APR 1999"))
    assert('1999-01-01' == normalize_date("1999"))

    assert('1999-04-01' == normalize_date("1APR1999"))
    assert('2001-04-01' == normalize_date("1APR2001"))

    assert('1999-04-18' == normalize_date("1999-04-11+7"))
    assert('1999-04-18' == normalize_date("1999-04-11 7"))
    assert('1999-04-01' == normalize_date("1 apr 1999"))
    assert('1999-04-11' == normalize_date("11 apr 1999"))
    assert('1999-04-11' == normalize_date("11 Apr 1999"))
    assert('1999-04-11' == normalize_date("11-apr-1999"))
    assert('1999-04-11' == normalize_date("11 April 1999"))
    assert('1999-04-11' == normalize_date("11 APRIL 1999"))
    assert('1999-04-11' == normalize_date("11 april 1999"))
    assert('1999-04-11' == normalize_date("11 aprick 1999"))
    assert('1999-04-11' == normalize_date("APR 11, 1999"))
    assert('1999-04-11' == normalize_date("4/11/1999"))
    assert('1999-04-11' == normalize_date("4-11-1999"))
    assert('1999-04-11' == normalize_date("1999-4-11"))
    assert('1999-04-11' == normalize_date("19990411"))

    assert('1999-01-01' == normalize_date("1 Jan 1999"))
    assert('1999-02-01' == normalize_date("1 Feb 1999"))
    assert('1999-03-01' == normalize_date("1 Mar 1999"))
    assert('1999-04-01' == normalize_date("1 Apr 1999"))
    assert('1999-05-01' == normalize_date("1 May 1999"))
    assert('1999-06-01' == normalize_date("1 Jun 1999"))
    assert('1999-07-01' == normalize_date("1 Jul 1999"))
    assert('1999-08-01' == normalize_date("1 Aug 1999"))
    assert('1999-09-01' == normalize_date("1 Sep 1999"))
    assert('1999-10-01' == normalize_date("1 Oct 1999"))
    assert('1999-11-01' == normalize_date("1 Nov 1999"))
    assert('1999-12-01' == normalize_date("1 Dec 1999"))

    assert('1999-04-30' == normalize_date("1999-4-30"))
    assert('2000-02-29' == normalize_date("29 FEB 2000"))
    assert('2001-02-28' == normalize_date("28 FEB 2001"))
    assert('2004-02-29' == normalize_date("29 FEB 2004"))
    assert('2100-02-28' == normalize_date("28 FEB 2100"))
    assert('1900-02-28' == normalize_date("28 FEB 1900"))

    def assertError(val):
        try:
            normalize_date(val)
        except (TypeError,ValueError):
            return
        raise ValueError("type error expected", val)

    assertError("2000-13-11")
    assertError("APR 99")
    assertError("29 FEB 1900")
    assertError("29 FEB 2100")
    assertError("29 FEB 2001")
    assertError("1999-4-31")
    assertError("APR 99")
    assertError("20301")
    assertError("020301")
    assertError("1APR99")
    assertError("1APR01")
    assertError("1 APR 99")
    assertError("1 APR 01")
    assertError("11/5/01")




import email
import io
import socket

from paste.httpserver import LimitedLengthFile, WSGIHandler, serve


class MockServer:
    server_address = ('127.0.0.1', 80)


class MockSocket:
    def makefile(self, mode, bufsize):
        return io.StringIO()


def test_environ():
    mock_socket = MockSocket()
    mock_client_address = '1.2.3.4'
    mock_server = MockServer()

    wsgi_handler = WSGIHandler(mock_socket, mock_client_address, mock_server)
    wsgi_handler.command = 'GET'
    wsgi_handler.path = '/path'
    wsgi_handler.request_version = 'HTTP/1.0'
    wsgi_handler.headers = email.message_from_string('Host: mywebsite')

    wsgi_handler.wsgi_setup()

    assert wsgi_handler.wsgi_environ['HTTP_HOST'] == 'mywebsite'


def test_environ_with_multiple_values():
    mock_socket = MockSocket()
    mock_client_address = '1.2.3.4'
    mock_server = MockServer()

    wsgi_handler = WSGIHandler(mock_socket, mock_client_address, mock_server)
    wsgi_handler.command = 'GET'
    wsgi_handler.path = '/path'
    wsgi_handler.request_version = 'HTTP/1.0'
    wsgi_handler.headers = email.message_from_string('Host: host1\nHost: host2')

    wsgi_handler.wsgi_setup()

    assert wsgi_handler.wsgi_environ['HTTP_HOST'] == 'host1,host2'


def test_limited_length_file():
    backing = io.BytesIO(b'0123456789')
    f = LimitedLengthFile(backing, 9)
    assert f.tell() == 0
    assert f.read() == b'012345678'
    assert f.tell() == 9
    assert f.read() == b''

def test_limited_length_file_tell_on_socket():
    backing_read, backing_write = socket.socketpair()
    f = LimitedLengthFile(backing_read.makefile('rb'), 10)
    backing_write.send(b'0123456789')
    backing_write.close()
    assert f.tell() == 0
    assert f.read(1) == b'0'
    assert f.tell() == 1
    assert f.read() == b'123456789'
    assert f.tell() == 10
    backing_read.close()


def test_address_family_v4():
    #ipv4
    app = None
    host = '127.0.0.1'
    port = '9090'
    
    svr = serve(app, host=host, port=port, start_loop=False, use_threadpool=False)

    af = svr.address_family
    addr = svr.server_address
    p = svr.server_port
    
    svr.server_close()

    assert (af == socket.AF_INET)
    assert (addr[0] == '127.0.0.1')
    assert (str(p) == port)


def test_address_family_v4_host_and_port():
    #ipv4
    app = None
    host = '127.0.0.1:9091'
    
    svr = serve(app, host=host, start_loop=False, use_threadpool=False)
    
    af = svr.address_family
    addr = svr.server_address
    p = svr.server_port
    
    svr.server_close()
    
    assert (af == socket.AF_INET)
    assert (addr[0] == '127.0.0.1')
    assert (str(p) == '9091')

def test_address_family_v6():
    #ipv6
    app = None
    host = '[::1]'
    port = '9090'
    
    try:
        svr = serve(app, host=host, port=port, start_loop=False, use_threadpool=False)
        
        af = svr.address_family
        addr = svr.server_address
        p = svr.server_port

        svr.server_close()
        
        assert (af == socket.AF_INET6)
        assert (addr[0] == '::1')
        assert (str(p) == port)
    except (socket.error, OSError) as err:
        # v6 support not available in this OS, pass the test
        assert True



from paste.debug.debugapp import SimpleApplication, SlowConsumer
from paste.fixture import TestApp
from paste.wsgiwrappers import WSGIRequest


def test_fixture():
    app = TestApp(SimpleApplication())
    res = app.get('/', params={'a': ['1', '2']})
    assert (res.request.environ['QUERY_STRING'] ==
            'a=1&a=2')
    res = app.put('/')
    assert (res.request.environ['REQUEST_METHOD'] ==
            'PUT')
    res = app.delete('/')
    assert (res.request.environ['REQUEST_METHOD'] ==
            'DELETE')
    class FakeDict:
        def items(self):
            return [('a', '10'), ('a', '20')]
    res = app.post('/params', params=FakeDict())

    # test multiple cookies in one request
    app.cookies['one'] = 'first';
    app.cookies['two'] = 'second';
    app.cookies['three'] = '';
    res = app.get('/')
    hc = res.request.environ['HTTP_COOKIE'].split('; ');
    assert ('one=first' in hc)
    assert ('two=second' in hc)
    assert ('three=' in hc)


def test_fixture_form():
    app = TestApp(SlowConsumer())
    res = app.get('/')
    form = res.forms[0]
    assert 'file' in form.fields
    assert form.action == ''


def test_fixture_form_end():
    def response(environ, start_response):
        body = b"<html><body><form>sm\xc3\xb6rebr\xc3\xb6</form></body></html>"
        start_response("200 OK", [('Content-Type', 'text/html'),
                                  ('Content-Length', str(len(body)))])
        return [body]
    TestApp(response).get('/')

def test_params_and_upload_files():
    class PostApp:
        def __call__(self, environ, start_response):
            start_response("204 No content", [])
            self.request = WSGIRequest(environ)
            return [b'']
    post_app = PostApp()
    app = TestApp(post_app)
    app.post(
        '/',
        params={'param1': 'a', 'param2': 'b'},
        upload_files=[
            ('file1', 'myfile.txt', b'data1'),
            ('file2', b'yourfile.txt', b'data2'),
        ],
    )
    params = post_app.request.params
    assert len(params) == 4
    assert params['param1'] == 'a'
    assert params['param2'] == 'b'
    assert params['file1'].value == b'data1'
    assert params['file1'].filename == 'myfile.txt'
    assert params['file2'].value == b'data2'
    assert params['file2'].filename == 'yourfile.txt'

def test_unicode_path():
    app = TestApp(SimpleApplication())
    app.get(u"/?")
    app.post(u"/?")
    app.put(u"/?")
    app.delete(u"/?")



# (c) 2010 Ch. Zwerschke and contributors
# This module is part of the Python Paste Project and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from paste.util.mimeparse import (
    best_match,
    desired_matches,
    fitness_and_quality_parsed,
    parse_media_range,
    parse_mime_type,
    quality,
    quality_parsed,
)

def test_parse_mime_type():
    parse = parse_mime_type
    assert parse('*/*') == ('*', '*', {})
    assert parse('text/html') == ('text', 'html', {})
    assert parse('audio/*; q=0.2') == ('audio', '*', {'q': '0.2'})
    assert parse('text/x-dvi;level=1') == ('text', 'x-dvi', {'level': '1'})
    assert parse('image/gif; level=2; q=0.4') == (
        'image', 'gif', {'level': '2', 'q': '0.4'})
    assert parse('application/xhtml;level=3;q=0.5') == (
        'application', 'xhtml', {'level': '3', 'q': '0.5'})
    assert parse('application/xml') == ('application', 'xml', {})
    assert parse('application/xml;q=1') == ('application', 'xml', {'q': '1'})
    assert parse('application/xml ; q=1;b=other') == (
        'application', 'xml', {'q': '1', 'b': 'other'})
    assert parse('application/xml ; q=2;b=other') == (
        'application', 'xml', {'q': '2', 'b': 'other'})
    assert parse('application/xhtml;q=0.5') == (
        'application', 'xhtml', {'q': '0.5'})
    assert parse('application/xhtml;q=0.5;ver=1.2') == (
        'application', 'xhtml', {'q': '0.5', 'ver': '1.2'})

def test_parse_illformed_mime_type():
    parse = parse_mime_type
    assert parse('*') == ('*', '*', {})
    assert parse('text') == ('text', '*', {})
    assert parse('text/') == ('text', '*', {})
    assert parse('/plain') == ('*', 'plain', {})
    assert parse('/') == ('*', '*', {})
    assert parse('text/plain;') == ('text', 'plain', {})
    assert parse(';q=0.5') == ('*', '*', {'q': '0.5'})
    assert parse('*; q=.2') == ('*', '*', {'q': '.2'})
    assert parse('image; q=.7; level=3') == (
        'image', '*', {'q': '.7', 'level': '3'})
    assert parse('*;q=1') == ('*', '*', {'q': '1'})
    assert parse('*;q=') == ('*', '*', {})
    assert parse('*;=0.5') == ('*', '*', {})
    assert parse('*;q=foobar') == ('*', '*', {'q': 'foobar'})
    assert parse('image/gif; level=2; q=2') == (
        'image', 'gif', {'level': '2', 'q': '2'})
    assert parse('application/xml;q=') == ('application', 'xml', {})
    assert parse('application/xml ;q=') == ('application', 'xml', {})
    assert parse(' *; q =;') == ('*', '*', {})
    assert parse(' *; q=.2') == ('*', '*', {'q': '.2'})

def test_parse_media_range():
    parse = parse_media_range
    assert parse('application/*;q=0.5') == ('application', '*', {'q': '0.5'})
    assert parse('text/plain') == ('text', 'plain', {'q': '1'})
    assert parse('*') == ('*', '*', {'q': '1'})
    assert parse(';q=0.5') == ('*', '*', {'q': '0.5'})
    assert parse('*;q=0.5') == ('*', '*', {'q': '0.5'})
    assert parse('*;q=1') == ('*', '*', {'q': '1'})
    assert parse('*;q=') == ('*', '*', {'q': '1'})
    assert parse('*;q=-1') == ('*', '*', {'q': '1'})
    assert parse('*;q=foobar') == ('*', '*', {'q': '1'})
    assert parse('*;q=0.0001') == ('*', '*', {'q': '0.0001'})
    assert parse('*;q=1000.0') == ('*', '*', {'q': '1'})
    assert parse('*;q=0') == ('*', '*', {'q': '0'})
    assert parse('*;q=0.0000') == ('*', '*', {'q': '0.0000'})
    assert parse('*;q=1.0001') == ('*', '*', {'q': '1'})
    assert parse('*;q=2') == ('*', '*', {'q': '1'})
    assert parse('*;q=1e3') == ('*', '*', {'q': '1'})
    assert parse('image/gif; level=2') == (
        'image', 'gif', {'level': '2', 'q': '1'})
    assert parse('image/gif; level=2; q=0.5') == (
        'image', 'gif', {'level': '2', 'q': '0.5'})
    assert parse('image/gif; level=2; q=2') == (
        'image', 'gif', {'level': '2', 'q': '1'})
    assert parse('application/xml') == ('application', 'xml', {'q': '1'})
    assert parse('application/xml;q=1') == ('application', 'xml', {'q': '1'})
    assert parse('application/xml;q=') == ('application', 'xml', {'q': '1'})
    assert parse('application/xml ;q=') == ('application', 'xml', {'q': '1'})
    assert parse('application/xml ; q=1;b=other') == (
        'application', 'xml', {'q': '1', 'b': 'other'})
    assert parse('application/xml ; q=2;b=other') == (
        'application', 'xml', {'q': '1', 'b': 'other'})
    assert parse(' *; q =;') == ('*', '*', {'q': '1'})
    assert parse(' *; q=.2') == ('*', '*', {'q': '.2'})

def test_fitness_and_quality_parsed():
    faq = fitness_and_quality_parsed
    assert faq('*/*;q=0.7', [
        ('foo', 'bar', {'q': '0.5'})]) == (0, 0.5)
    assert faq('foo/*;q=0.7', [
        ('foo', 'bar', {'q': '0.5'})]) == (100, 0.5)
    assert faq('*/bar;q=0.7', [
        ('foo', 'bar', {'q': '0.5'})]) == (10, 0.5)
    assert faq('foo/bar;q=0.7', [
        ('foo', 'bar', {'q': '0.5'})]) == (110, 0.5)
    assert faq('text/html;q=0.7', [
        ('foo', 'bar', {'q': '0.5'})]) == (-1, 0)
    assert faq('text/html;q=0.7', [
        ('text', 'bar', {'q': '0.5'})]) == (-1, 0)
    assert faq('text/html;q=0.7', [
        ('foo', 'html', {'q': '0.5'})]) == (-1, 0)
    assert faq('text/html;q=0.7', [
        ('text', '*', {'q': '0.5'})]) == (100, 0.5)
    assert faq('text/html;q=0.7', [
        ('*', 'html', {'q': '0.5'})]) == (10, 0.5)
    assert faq('text/html;q=0.7', [
        ('*', '*', {'q': '0'}), ('text', 'html', {'q': '0.5'})]) == (110, 0.5)
    assert faq('text/html;q=0.7', [
        ('*', '*', {'q': '0.5'}), ('audio', '*', {'q': '0'})]) == (0, 0.5)
    assert faq('audio/mp3;q=0.7', [
        ('*', '*', {'q': '0'}), ('audio', '*', {'q': '0.5'})]) == (100, 0.5)
    assert faq('*/mp3;q=0.7', [
        ('foo', 'mp3', {'q': '0.5'}), ('audio', '*', {'q': '0'})]) == (10, 0.5)
    assert faq('audio/mp3;q=0.7', [
        ('audio', 'ogg', {'q': '0'}), ('*', 'mp3', {'q': '0.5'})]) == (10, 0.5)
    assert faq('audio/mp3;q=0.7', [
        ('*', 'ogg', {'q': '0'}), ('*', 'mp3', {'q': '0.5'})]) == (10, 0.5)
    assert faq('text/html;q=0.7', [
        ('text', 'plain', {'q': '0'}),
        ('plain', 'html', {'q': '0'}),
        ('text', 'html', {'q': '0.5'}),
        ('html', 'text', {'q': '0'})]) == (110, 0.5)
    assert faq('text/html;q=0.7;level=2', [
        ('plain', 'html', {'q': '0', 'level': '2'}),
        ('text', '*', {'q': '0.5', 'level': '3'}),
        ('*', 'html', {'q': '0.5', 'level': '2'}),
        ('image', 'gif', {'q': '0.5', 'level': '2'})]) == (100, 0.5)
    assert faq('text/html;q=0.7;level=2', [
        ('text', 'plain', {'q': '0'}), ('text', 'html', {'q': '0'}),
        ('text', 'plain', {'q': '0', 'level': '2'}),
        ('text', 'html', {'q': '0.5', 'level': '2'}),
        ('*', '*', {'q': '0', 'level': '2'}),
        ('text', 'html', {'q': '0', 'level': '3'})]) == (111, 0.5)
    assert faq('text/html;q=0.7;level=2;opt=3', [
        ('text', 'html', {'q': '0'}),
        ('text', 'html', {'q': '0', 'level': '2'}),
        ('text', 'html', {'q': '0', 'opt': '3'}),
        ('*', '*', {'q': '0', 'level': '2', 'opt': '3'}),
        ('text', 'html', {'q': '0', 'level': '3', 'opt': '3'}),
        ('text', 'html', {'q': '0.5', 'level': '2', 'opt': '3'}),
        ('*', '*', {'q': '0', 'level': '3', 'opt': '3'})]) == (112, 0.5)

def test_quality_parsed():
    qp = quality_parsed
    assert qp('image/gif;q=0.7', [('image', 'jpg', {'q': '0.5'})]) == 0
    assert qp('image/gif;q=0.7', [('image', '*', {'q': '0.5'})]) == 0.5
    assert qp('audio/mp3;q=0.7;quality=100', [
        ('*', '*', {'q': '0', 'quality': '100'}),
        ('audio', '*', {'q': '0', 'quality': '100'}),
        ('*', 'mp3', {'q': '0', 'quality': '100'}),
        ('audio', 'mp3', {'q': '0', 'quality': '50'}),
        ('audio', 'mp3', {'q': '0.5', 'quality': '100'}),
        ('audio', 'mp3', {'q': '0.5'})]) == 0.5

def test_quality():
    assert quality('text/html',
        'text/*;q=0.3, text/html;q=0.75, text/html;level=1,'
        ' text/html;level=2;q=0.4, */*;q=0.5') == 0.75
    assert quality('text/html;level=2',
        'text/*;q=0.3, text/html;q=0.7, text/html;level=1,'
        ' text/html;level=2;q=0.4, */*;q=0.5') == 0.4
    assert quality('text/plain',
        'text/*;q=0.25, text/html;q=0.7, text/html;level=1,'
        ' text/html;level=2;q=0.4, */*;q=0.5') == 0.25
    assert quality('plain/text',
        'text/*;q=0.3, text/html;q=0.7, text/html;level=1,'
        ' text/html;level=2;q=0.4, */*;q=0.5') == 0.5
    assert quality('text/html;level=1',
        'text/*;q=0.3, text/html;q=0.7, text/html;level=1,'
        ' text/html;level=2;q=0.4, */*;q=0.5') == 1
    assert quality('image/jpeg',
        'text/*;q=0.3, text/html;q=0.7, text/html;level=1,'
        ' text/html;level=2;q=0.4, */*;q=0.5') == 0.5
    assert quality('text/html;level=2',
        'text/*;q=0.3, text/html;q=0.7, text/html;level=1,'
        ' text/html;level=2;q=0.375, */*;q=0.5') == 0.375
    assert quality('text/html;level=3',
        'text/*;q=0.3, text/html;q=0.75, text/html;level=1,'
        ' text/html;level=2;q=0.4, */*;q=0.5') == 0.75

def test_best_match():
    bm = best_match
    assert bm([], '*/*') == ''
    assert bm(['application/xbel+xml', 'text/xml'],
        'text/*;q=0.5,*/*; q=0.1') == 'text/xml'
    assert bm(['application/xbel+xml', 'audio/mp3'],
        'text/*;q=0.5,*/*; q=0.1') == 'application/xbel+xml'
    assert bm(['application/xbel+xml', 'audio/mp3'],
        'text/*;q=0.5,*/mp3; q=0.1') == 'audio/mp3'
    assert bm(['application/xbel+xml', 'text/plain', 'text/html'],
        'text/*;q=0.5,*/plain; q=0.1') == 'text/plain'
    assert bm(['application/xbel+xml', 'text/html', 'text/xhtml'],
        'text/*;q=0.1,*/xhtml; q=0.5') == 'text/html'
    assert bm(['application/xbel+xml', 'text/html', 'text/xhtml'],
        '*/html;q=0.1,*/xhtml; q=0.5') == 'text/xhtml'
    assert bm(['application/xbel+xml', 'application/xml'],
        'application/xbel+xml') == 'application/xbel+xml'
    assert bm(['application/xbel+xml', 'application/xml'],
        'application/xbel+xml; q=1') == 'application/xbel+xml'
    assert bm(['application/xbel+xml', 'application/xml'],
        'application/xml; q=1') == 'application/xml'
    assert bm(['application/xbel+xml', 'application/xml'],
        'application/*; q=1') == 'application/xbel+xml'
    assert bm(['application/xbel+xml', 'application/xml'],
        '*/*, application/xml') == 'application/xml'
    assert bm(['application/xbel+xml', 'text/xml'],
        'text/*;q=0.5,*/*; q=0.1') == 'text/xml'
    assert bm(['application/xbel+xml', 'text/xml'],
        'text/html,application/atom+xml; q=0.9') == ''
    assert bm(['application/json', 'text/html'],
        'application/json, text/javascript, */*') == 'application/json'
    assert bm(['application/json', 'text/html'],
        'application/json, text/html;q=0.9') == 'application/json'
    assert bm(['image/*', 'application/xml'], 'image/png') == 'image/*'
    assert bm(['image/*', 'application/xml'], 'image/*') == 'image/*'

def test_illformed_best_match():
    bm = best_match
    assert bm(['image/png', 'image/jpeg', 'image/gif', 'text/html'],
        'text/html, image/gif, image/jpeg, *; q=.2, */*; q=.2') == 'image/jpeg'
    assert bm(['image/png', 'image/jpg', 'image/tif', 'text/html'],
        'text/html, image/gif, image/jpeg, *; q=.2, */*; q=.2') == 'text/html'
    assert bm(['image/png', 'image/jpg', 'image/tif', 'audio/mp3'],
        'text/html, image/gif, image/jpeg, *; q=.2, */*; q=.2') == 'image/png'

def test_sorted_match():
    dm = desired_matches
    assert dm(['text/html', 'application/xml'],
        'text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,'
        'text/plain;q=0.8,image/png') == ['text/html', 'application/xml']
    assert dm(['text/html', 'application/xml'],
        'application/xml,application/json') == ['application/xml']
    assert dm(['text/xhtml', 'text/plain', 'application/xhtml'],
        'text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,'
        'text/plain;q=0.8,image/png') == ['text/plain']



# Procedure to release a new version:
#
# - update version in setup.py (__version__)
# - update changelog: docs/news.txt
# - commit and push to git
# - make release
#
# The final step will release to pypi and to Github

__version__ = '3.10.1'

from setuptools import setup, find_packages
import sys, os
sys.path.append(os.path.join(os.path.dirname(__file__),
                                'paste', 'util'))
import finddata

with open("README.rst") as fp:
    README = fp.read()

setup(name="Paste",
      version=__version__,
      python_requires=">=3",
      description="Tools for using a Web Server Gateway Interface stack",
      long_description=README,
      classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Server",
        "Framework :: Paste",
        ],
      keywords='web application server wsgi',
      author="Chris Dent",
      author_email="chris.dent@gmail.com",
      url="https://pythonpaste.readthedocs.io/",
      project_urls={
        "Source": "https://github.com/pasteorg/paste",
        "Bug Tracker": "https://github.com/pasteorg/paste/issues",
        "Documentation": "https://pythonpaste.readthedocs.io"
      },
      license="MIT",
      packages=find_packages(exclude=['ez_setup', 'examples', 'packages', 'tests*']),
      package_data=finddata.find_package_data(
          exclude_directories=finddata.standard_exclude_directories + ('tests',)),
      namespace_packages=['paste'],
      zip_safe=False,
      install_requires=[
        'setuptools',  # pkg_resources
        ],
      extras_require={
        'subprocess': [],
        'hotshot': [],
        'Flup': ['flup'],
        'Paste': [],
        'openid': ['python-openid'],
        },
      entry_points="""
      [paste.app_factory]
      cgi = paste.cgiapp:make_cgi_application [subprocess]
      static = paste.urlparser:make_static
      pkg_resources = paste.urlparser:make_pkg_resources
      urlparser = paste.urlparser:make_url_parser
      proxy = paste.proxy:make_proxy
      test = paste.debug.debugapp:make_test_app
      test_slow = paste.debug.debugapp:make_slow_app
      transparent_proxy = paste.proxy:make_transparent_proxy
      watch_threads = paste.debug.watchthreads:make_watch_threads

      [paste.composite_factory]
      urlmap = paste.urlmap:urlmap_factory
      cascade = paste.cascade:make_cascade

      [paste.filter_app_factory]
      error_catcher = paste.exceptions.errormiddleware:make_error_middleware
      cgitb = paste.cgitb_catcher:make_cgitb_middleware
      flup_session = paste.flup_session:make_session_middleware [Flup]
      gzip = paste.gzipper:make_gzip_middleware
      httpexceptions = paste.httpexceptions:make_middleware
      lint = paste.lint:make_middleware
      printdebug = paste.debug.prints:PrintDebugMiddleware
      profile = paste.debug.profile:make_profile_middleware [hotshot]
      recursive = paste.recursive:make_recursive_middleware
      # This isn't good enough to deserve the name egg:Paste#session:
      paste_session = paste.session:make_session_middleware
      wdg_validate = paste.debug.wdg_validate:make_wdg_validate_middleware [subprocess]
      evalerror = paste.evalexception.middleware:make_eval_exception
      auth_tkt = paste.auth.auth_tkt:make_auth_tkt_middleware
      auth_basic = paste.auth.basic:make_basic
      auth_digest = paste.auth.digest:make_digest
      auth_form = paste.auth.form:make_form
      grantip = paste.auth.grantip:make_grantip
      openid = paste.auth.open_id:make_open_id_middleware [openid]
      pony = paste.pony:make_pony
      cowbell = paste.cowbell:make_cowbell
      errordocument = paste.errordocument:make_errordocument
      auth_cookie = paste.auth.cookie:make_auth_cookie
      translogger = paste.translogger:make_filter
      config = paste.config:make_config_filter
      registry = paste.registry:make_registry_manager

      [paste.server_runner]
      http = paste.httpserver:server_runner
      """,
      )



# (c) 2005 Ian Bicking, Clark C. Evans and contributors
# This module is part of the Python Paste Project and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
"""
WSGI Exception Middleware

Regression Test Suite
"""

import pytest

from paste.httpexceptions import (
    get_exception,
    HTTPBadRequest,
    HTTPClientError,
    HTTPError,
    HTTPException,
    HTTPExceptionHandler,
    HTTPFound,
    HTTPInternalServerError,
    HTTPMovedPermanently,
    HTTPNotFound,
    HTTPNotImplemented,
    HTTPRedirection,
    HTTPServerError,
    HTTPTemporaryRedirect,
    HTTPUseProxy,
)
from paste.response import header_value


def test_HTTPMove():
    """ make sure that location is a mandatory attribute of Redirects """
    pytest.raises(AssertionError, HTTPFound)
    pytest.raises(AssertionError, HTTPTemporaryRedirect,
                   headers=[('l0cation','/bing')])
    assert isinstance(HTTPMovedPermanently("This is a message",
                          headers=[('Location','/bing')])
                     ,HTTPRedirection)
    assert isinstance(HTTPUseProxy(headers=[('LOCATION','/bing')])
                     ,HTTPRedirection)
    assert isinstance(HTTPFound('/foobar'),HTTPRedirection)

def test_badapp():
    """ verify that the middleware handles previously-started responses """
    def badapp(environ, start_response):
        start_response("200 OK",[])
        raise HTTPBadRequest("Do not do this at home.")
    newapp = HTTPExceptionHandler(badapp)
    assert b'Bad Request' in b''.join(newapp({'HTTP_ACCEPT': 'text/html'},
                                             (lambda a, b, c=None: None)))

def test_unicode():
    """ verify unicode output """
    tstr = u"\0xCAFE"
    def badapp(environ, start_response):
        start_response("200 OK",[])
        raise HTTPBadRequest(tstr)
    newapp = HTTPExceptionHandler(badapp)
    assert tstr.encode("utf-8") in b''.join(newapp({'HTTP_ACCEPT':
                                         'text/html'},
                                         (lambda a, b, c=None: None)))
    assert tstr.encode("utf-8") in b''.join(newapp({'HTTP_ACCEPT':
                                         'text/plain'},
                                         (lambda a, b, c=None: None)))

def test_template():
    """ verify that html() and plain() output methods work """
    e = HTTPInternalServerError()
    e.template = 'A %(ping)s and <b>%(pong)s</b> message.'
    assert str(e).startswith("500 Internal Server Error")
    assert e.plain({'ping': 'fun', 'pong': 'happy'}) == (
        '500 Internal Server Error\r\n'
        'A fun and happy message.\r\n')
    assert '<p>A fun and <b>happy</b> message.</p>' in \
           e.html({'ping': 'fun', 'pong': 'happy'})

def test_redapp():
    """ check that redirect returns the correct, expected results """
    saved = []
    def saveit(status, headers, exc_info = None):
        saved.append((status,headers))
    def redapp(environ, start_response):
        raise HTTPFound("/bing/foo")
    app = HTTPExceptionHandler(redapp)
    result = list(app({'HTTP_ACCEPT': 'text/html'},saveit))
    assert b'<a href="/bing/foo">' in result[0]
    assert "302 Found" == saved[0][0]
    assert "text/html; charset=utf8" == header_value(saved[0][1], 'content-type')
    assert "/bing/foo" == header_value(saved[0][1],'location')
    result = list(app({'HTTP_ACCEPT': 'text/plain'},saveit))
    assert "text/plain; charset=utf8" == header_value(saved[1][1],'content-type')
    assert "/bing/foo" == header_value(saved[1][1],'location')

def test_misc():
    assert get_exception(301) == HTTPMovedPermanently
    redirect = HTTPFound("/some/path")
    assert isinstance(redirect,HTTPException)
    assert isinstance(redirect,HTTPRedirection)
    assert not isinstance(redirect,HTTPError)
    notfound = HTTPNotFound()
    assert isinstance(notfound,HTTPException)
    assert isinstance(notfound,HTTPError)
    assert isinstance(notfound,HTTPClientError)
    assert not isinstance(notfound,HTTPServerError)
    notimpl = HTTPNotImplemented()
    assert isinstance(notimpl,HTTPException)
    assert isinstance(notimpl,HTTPError)
    assert isinstance(notimpl,HTTPServerError)
    assert not isinstance(notimpl,HTTPClientError)





def application(environ, start_response):
    start_response('200 OK', [('Content-type', 'text/html')])
    body = 'index: %s' % environ['app.user']
    body = body.encode('ascii')
    return [body]




# -*- coding: utf-8 -*-
# (c) 2007 Philip Jenvey; written for Paste (http://pythonpaste.org)
# Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php
import io
from paste.fixture import TestApp
from paste.wsgiwrappers import WSGIRequest, WSGIResponse
from paste.util.field_storage import FieldStorage

class AssertApp:
    def __init__(self, assertfunc):
        self.assertfunc = assertfunc

    def __call__(self, environ, start_response):
        start_response('200 OK', [('Content-type','text/plain')])
        self.assertfunc(environ)
        return [b'Passed']

no_encoding = object()
def valid_name(name, encoding=no_encoding, post=False):
    def assert_valid_name(environ):
        if encoding is not no_encoding:
            WSGIRequest.defaults._push_object(dict(content_type='text/html',
                                                   charset=encoding))
        try:
            request = WSGIRequest(environ)
            if post:
                params = request.POST
            else:
                params = request.GET
            assert params['name'] == name
            assert request.params['name'] == name
        finally:
            if encoding is not no_encoding:
                WSGIRequest.defaults._pop_object()
    return assert_valid_name

def test_wsgirequest_charset():
    # Jose, 'José'
    app = TestApp(AssertApp(assertfunc=valid_name(u'José', encoding='UTF-8')))
    res = app.get('/?name=Jos%C3%A9')

    # Tanaka, '田中'
    app = TestApp(AssertApp(assertfunc=valid_name(u'田中', encoding='UTF-8')))
    res = app.get('/?name=%E7%94%B0%E4%B8%AD')

    # Nippon (Japan), '日本'
    app = TestApp(AssertApp(assertfunc=valid_name(u'日本', encoding='UTF-8',
                                                  post=True)))
    res = app.post('/', params=dict(name='日本'))

    # WSGIRequest will determine the charset from the Content-Type header when
    # unicode is expected.
    # No encoding specified: not expecting unicode
    app = TestApp(AssertApp(assertfunc=valid_name('日本', post=True)))
    content_type = 'application/x-www-form-urlencoded; charset=%s'
    res = app.post('/', params=dict(name='日本'),
                   headers={'content-type': content_type % 'UTF-8'})

    # Encoding specified: expect unicode. Shiftjis is the default encoding, but
    # params become UTF-8 because the browser specified so
    app = TestApp(AssertApp(assertfunc=valid_name(u'日本', post=True,
                                                  encoding='shiftjis')))
    res = app.post('/', params=dict(name='日本'),
                   headers={'content-type': content_type % 'UTF-8'})

    # Browser did not specify: parse params as the fallback shiftjis
    app = TestApp(AssertApp(assertfunc=valid_name(u'日本', post=True,
                                                  encoding='shiftjis')))
    res = app.post('/', params=dict(name=u'日本'.encode('shiftjis')))

def test_wsgirequest_charset_fileupload():
    def handle_fileupload(environ, start_response):
        start_response('200 OK', [('Content-type','text/plain')])
        request = WSGIRequest(environ)

        assert len(request.POST) == 1
        assert isinstance(request.POST.keys()[0], str)
        fs = request.POST['thefile']
        assert isinstance(fs, FieldStorage)
        assert isinstance(fs.filename, str)
        assert fs.filename == '寿司.txt'
        assert fs.value == b'Sushi'

        request.charset = 'UTF-8'
        assert len(request.POST) == 1
        assert isinstance(request.POST.keys()[0], str)
        fs = request.POST['thefile']
        assert isinstance(fs, FieldStorage)
        assert isinstance(fs.filename, str)
        assert fs.filename == u'寿司.txt'
        assert fs.value == b'Sushi'

        request.charset = None
        assert fs.value == b'Sushi'
        return []

    app = TestApp(handle_fileupload)
    res = app.post('/', upload_files=[('thefile', u'寿司.txt'.encode('utf-8'), b'Sushi')])

def test_wsgiresponse_charset():
    response = WSGIResponse(mimetype='text/html; charset=UTF-8')
    assert response.content_type == 'text/html'
    assert response.charset == 'UTF-8'
    response.write(u'test')
    response.write(u'test2')
    response.write('test3')
    status, headers, content = response.wsgi_response()
    for data in content:
        assert isinstance(data, bytes)

    WSGIResponse.defaults._push_object(dict(content_type='text/html',
                                            charset='iso-8859-1'))
    try:
        response = WSGIResponse()
        response.write(u'test')
        response.write(u'test2')
        response.write('test3')
        status, headers, content = response.wsgi_response()
        for data in content:
            assert isinstance(data, bytes)
    finally:
        WSGIResponse.defaults._pop_object()

    # WSGIResponse will allow unicode to pass through when no charset is
    # set
    WSGIResponse.defaults._push_object(dict(content_type='text/html',
                                            charset=None))
    try:
        response = WSGIResponse(u'test')
        response.write(u'test1')
        status, headers, content = response.wsgi_response()
        for data in content:
            assert isinstance(data, str)
    finally:
        WSGIResponse.defaults._pop_object()

    WSGIResponse.defaults._push_object(dict(content_type='text/html',
                                            charset=''))
    try:
        response = WSGIResponse(u'test')
        response.write(u'test1')
        status, headers, content = response.wsgi_response()
        for data in content:
            assert isinstance(data, str)
    finally:
        WSGIResponse.defaults._pop_object()

def test_call_wsgiresponse():
    resp = WSGIResponse(b'some content', 'application/octet-stream')
    def sp(status, response_headers):
        assert status == '200 OK'
        assert sorted(response_headers) == [
            ('cache-control', 'no-cache'),
            ('content-type', 'application/octet-stream'),
        ]
    assert resp({}, sp) == [b'some content']
    f = io.BytesIO(b'some content')
    resp = WSGIResponse(f, 'application/octet-stream')
    assert list(resp({}, sp)) == [b'some content']
    f = io.BytesIO()
    resp = WSGIResponse(f, 'application/octet-stream')
    assert resp({'wsgi.file_wrapper': lambda x: x}, sp) is f



# (c) 2005 Ian Bicking, Clark C. Evans and contributors
# This module is part of the Python Paste Project and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
import time
import random
import os
import tempfile
from email.utils import parsedate_tz, mktime_tz

from paste import fileapp
from paste.fileapp import DataApp
from paste.fixture import TestApp

# NOTE(haypo): don't use string.letters because the order of lower and upper
# case letters changes when locale.setlocale() is called for the first time
LETTERS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'

def test_data():
    harness = TestApp(DataApp(b'mycontent'))
    res = harness.get("/")
    assert 'application/octet-stream' == res.header('content-type')
    assert '9' == res.header('content-length')
    assert "<Response 200 OK 'mycontent'>" == repr(res)
    harness.app.set_content(b"bingles")
    assert "<Response 200 OK 'bingles'>" == repr(harness.get("/"))

def test_cache():
    def build(*args,**kwargs):
        app = DataApp(b"SomeContent")
        app.cache_control(*args,**kwargs)
        return TestApp(app).get("/")
    res = build()
    assert 'public' == res.header('cache-control')
    assert not res.header('expires',None)
    res = build(private=True)
    assert 'private' == res.header('cache-control')
    assert mktime_tz(parsedate_tz(res.header('expires'))) < time.time()
    res = build(no_cache=True)
    assert 'no-cache' == res.header('cache-control')
    assert mktime_tz(parsedate_tz(res.header('expires'))) < time.time()
    res = build(max_age=60,s_maxage=30)
    assert 'public, max-age=60, s-maxage=30' == res.header('cache-control')
    expires = mktime_tz(parsedate_tz(res.header('expires')))
    assert expires > time.time()+58 and expires < time.time()+61
    res = build(private=True, max_age=60, no_transform=True, no_store=True)
    assert 'private, no-store, no-transform, max-age=60' == \
           res.header('cache-control')
    expires = mktime_tz(parsedate_tz(res.header('expires')))
    assert mktime_tz(parsedate_tz(res.header('expires'))) < time.time()

def test_disposition():
    def build(*args,**kwargs):
        app = DataApp(b"SomeContent")
        app.content_disposition(*args,**kwargs)
        return TestApp(app).get("/")
    res = build()
    assert 'attachment' == res.header('content-disposition')
    assert 'application/octet-stream' == res.header('content-type')
    res = build(filename="bing.txt")
    assert 'attachment; filename="bing.txt"' == \
            res.header('content-disposition')
    assert 'text/plain' == res.header('content-type')
    res = build(inline=True)
    assert 'inline' == res.header('content-disposition')
    assert 'application/octet-stream' == res.header('content-type')
    res = build(inline=True, filename="/some/path/bing.txt")
    assert 'inline; filename="bing.txt"' == \
            res.header('content-disposition')
    assert 'text/plain' == res.header('content-type')
    try:
       res = build(inline=True,attachment=True)
    except AssertionError:
        pass
    else:
        assert False, "should be an exception"

def test_modified():
    harness = TestApp(DataApp(b'mycontent'))
    res = harness.get("/")
    assert "<Response 200 OK 'mycontent'>" == repr(res)
    last_modified = res.header('last-modified')
    res = harness.get("/",headers={'if-modified-since': last_modified})
    assert "<Response 304 Not Modified ''>" == repr(res)
    res = harness.get("/",headers={'if-modified-since': last_modified + \
                                   '; length=1506'})
    assert "<Response 304 Not Modified ''>" == repr(res)
    res = harness.get("/",status=400,
            headers={'if-modified-since': 'garbage'})
    assert 400 == res.status and b"ill-formed timestamp" in res.body
    res = harness.get("/",status=400,
            headers={'if-modified-since':
                'Thu, 22 Dec 3030 01:01:01 GMT'})
    assert 400 == res.status and b"check your system clock" in res.body

def test_file():
    tempfile = "test_fileapp.%s.txt" % (random.random())
    content = LETTERS * 20
    content = content.encode('utf8')
    with open(tempfile, "wb") as fp:
        fp.write(content)
    try:
        app = fileapp.FileApp(tempfile)
        res = TestApp(app).get("/")
        assert len(content) == int(res.header('content-length'))
        assert 'text/plain' == res.header('content-type')
        assert content == res.body
        assert content == app.content  # this is cashed
        lastmod = res.header('last-modified')
        print("updating", tempfile)
        file = open(tempfile,"a+")
        file.write("0123456789")
        file.close()
        res = TestApp(app).get("/",headers={'Cache-Control': 'max-age=0'})
        assert len(content)+10 == int(res.header('content-length'))
        assert 'text/plain' == res.header('content-type')
        assert content + b"0123456789" == res.body
        assert app.content # we are still cached
        file = open(tempfile,"a+")
        file.write("X" * fileapp.CACHE_SIZE) # exceed the cashe size
        file.write("YZ")
        file.close()
        res = TestApp(app).get("/",headers={'Cache-Control': 'max-age=0'})
        newsize = fileapp.CACHE_SIZE + len(content)+12
        assert newsize == int(res.header('content-length'))
        assert newsize == len(res.body)
        assert res.body.startswith(content) and res.body.endswith(b'XYZ')
        assert not app.content # we are no longer cached
    finally:
        os.unlink(tempfile)

def test_dir():
    tmpdir = tempfile.mkdtemp()
    try:
        tmpfile = os.path.join(tmpdir, 'file')
        tmpsubdir = os.path.join(tmpdir, 'dir')
        fp = open(tmpfile, 'w')
        fp.write('abcd')
        fp.close()
        os.mkdir(tmpsubdir)
        try:
            app = fileapp.DirectoryApp(tmpdir)
            for path in ['/', '', '//', '/..', '/.', '/../..']:
                assert TestApp(app).get(path, status=403).status == 403, ValueError(path)
            for path in ['/~', '/foo', '/dir', '/dir/']:
                assert TestApp(app).get(path, status=404).status == 404, ValueError(path)
            assert TestApp(app).get('/file').body == b'abcd'
        finally:
            os.remove(tmpfile)
            os.rmdir(tmpsubdir)
    finally:
        os.rmdir(tmpdir)

def _excercize_range(build,content):
    # full content request, but using ranges'
    res = build("bytes=0-%d" % (len(content)-1))
    assert res.header('accept-ranges') == 'bytes'
    assert res.body == content
    assert res.header('content-length') == str(len(content))
    res = build("bytes=-%d" % (len(content)-1))
    assert res.body == content
    assert res.header('content-length') == str(len(content))
    res = build("bytes=0-")
    assert res.body == content
    assert res.header('content-length') == str(len(content))
    # partial content requests
    res = build("bytes=0-9", status=206)
    assert res.body == content[:10]
    assert res.header('content-length') == '10'
    res = build("bytes=%d-" % (len(content)-1), status=206)
    assert res.body == b'Z'
    assert res.header('content-length') == '1'
    res = build("bytes=%d-%d" % (3,17), status=206)
    assert res.body == content[3:18]
    assert res.header('content-length') == '15'

def test_range():
    content = LETTERS * 5
    content = content.encode('utf8')
    def build(range, status=206):
        app = DataApp(content)
        return TestApp(app).get("/",headers={'Range': range}, status=status)
    _excercize_range(build,content)
    build('bytes=0-%d' % (len(content)+1), 416)

def test_file_range():
    tempfile = "test_fileapp.%s.txt" % (random.random())
    content = LETTERS * (1+(fileapp.CACHE_SIZE // len(LETTERS)))
    content = content.encode('utf8')
    assert len(content) > fileapp.CACHE_SIZE
    with open(tempfile, "wb") as fp:
        fp.write(content)
    try:
        def build(range, status=206):
            app = fileapp.FileApp(tempfile)
            return TestApp(app).get("/",headers={'Range': range},
                                        status=status)
        _excercize_range(build,content)
        for size in (13,len(LETTERS), len(LETTERS)-1):
            fileapp.BLOCK_SIZE = size
            _excercize_range(build,content)
    finally:
        os.unlink(tempfile)

def test_file_cache():
    filename = os.path.join(os.path.dirname(__file__),
                            'urlparser_data', 'secured.txt')
    app = TestApp(fileapp.FileApp(filename))
    res = app.get('/')
    etag = res.header('ETag')
    last_mod = res.header('Last-Modified')
    res = app.get('/', headers={'If-Modified-Since': last_mod},
                  status=304)
    res = app.get('/', headers={'If-None-Match': etag},
                  status=304)
    res = app.get('/', headers={'If-None-Match': 'asdf'},
                  status=200)
    res = app.get('/', headers={'If-Modified-Since': 'Sat, 1 Jan 2005 12:00:00 GMT'},
                  status=200)
    res = app.get('/', headers={'If-Modified-Since': last_mod + '; length=100'},
                  status=304)
    res = app.get('/', headers={'If-Modified-Since': 'invalid date'},
                  status=400)

def test_methods():
    filename = os.path.join(os.path.dirname(__file__),
                            'urlparser_data', 'secured.txt')
    app = TestApp(fileapp.FileApp(filename))
    get_res = app.get('')
    res = app.get('', extra_environ={'REQUEST_METHOD': 'HEAD'})
    assert res.headers == get_res.headers
    assert not res.body
    app.post('', status=405) # Method Not Allowed

